from typing import Tuple, Set, Iterable, List


class TabbedDialogAction:
    def __init__(self, A_0: Object, A_1: IntPtr): ...
    def Invoke(self) -> None: ...
    def BeginInvoke(self, callback: AsyncCallback, obj: Object) -> IAsyncResult: ...
    def EndInvoke(self, result: IAsyncResult) -> None: ...


class TabbedDialogExtension:
    def __init__(self, userControl: UserControl, onOK: TabbedDialogAction): ...
    @property
    def Control(self) -> UserControl: ...
    @property
    def OnOKAction(self) -> TabbedDialogAction: ...
    @property
    def OnCancelAction(self) -> TabbedDialogAction: ...
    @OnCancelAction.setter
    def OnCancelAction(self, action: TabbedDialogAction) -> None: ...
    @property
    def OnRestoreDefaultsAction(self) -> TabbedDialogAction: ...
    @OnRestoreDefaultsAction.setter
    def OnRestoreDefaultsAction(self, action: TabbedDialogAction) -> None: ...
    def GetContextualHelp(self) -> ContextualHelp: ...
    def SetContextualHelp(self, contextualHelp: ContextualHelp) -> None: ...


class IDropHandler:
    def Execute(self, document: UIDocument, data: Object) -> None: ...


class IControllableDropHandler:
    def CanExecute(self, document: UIDocument, data: Object, dropViewId: ElementId) -> bool: ...


class RibbonPanel:
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, pName: str) -> None: ...
    @property
    def Title(self) -> str: ...
    @Title.setter
    def Title(self, pTitle: str) -> None: ...
    @property
    def Visible(self) -> bool: ...
    @Visible.setter
    def Visible(self, bVisible: bool) -> None: ...
    @property
    def Enabled(self) -> bool: ...
    @Enabled.setter
    def Enabled(self, bEnabled: bool) -> None: ...
    def GetItems(self) -> List[RibbonItem]: ...
    def AddSeparator(self) -> None: ...
    def AddItem(self, itemData: RibbonItemData) -> RibbonItem: ...
    @overload
    def AddStackedItems(self, item1: RibbonItemData, item2: RibbonItemData, item3: RibbonItemData) -> List[RibbonItem]: ...
    @overload
    def AddStackedItems(self, item1: RibbonItemData, item2: RibbonItemData) -> List[RibbonItem]: ...
    def AddSlideOut(self) -> None: ...
    def Equals(self, obj: Object) -> bool: ...


class UIDocument:
    def __init__(self, document: Document): ...
    def GetRevitUIFamilyLoadOptions() -> IFamilyLoadOptions: ...
    @property
    def Selection(self) -> Selection: ...
    def SaveAndClose(self) -> bool: ...
    @overload
    def ShowElements(self, elementIds: ICollection) -> None: ...
    @overload
    def ShowElements(self, elements: ElementSet) -> None: ...
    @overload
    def ShowElements(self, element: Element) -> None: ...
    @overload
    def ShowElements(self, elementId: ElementId) -> None: ...
    def RefreshActiveView(self) -> None: ...
    @property
    def ActiveView(self) -> View: ...
    @ActiveView.setter
    def ActiveView(self, view: View) -> None: ...
    @property
    def Document(self) -> Document: ...
    def RequestViewChange(self, view: View) -> None: ...
    @overload
    def PromptForFamilyInstancePlacement(self, familySymbol: FamilySymbol, options: PromptForFamilyInstancePlacementOptions) -> None: ...
    @overload
    def PromptForFamilyInstancePlacement(self, familySymbol: FamilySymbol) -> None: ...
    @property
    def Application(self) -> UIApplication: ...
    def GetOpenUIViews(self) -> List[UIView]: ...
    @property
    def ActiveGraphicalView(self) -> View: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetSketchGalleryOptions(self, familySymbol: FamilySymbol) -> List[SketchGalleryOptions]: ...
    def PromptToPlaceViewOnSheet(self, view: View, allowReplaceExistingSheetViewport: bool) -> None: ...
    def PostRequestForElementTypePlacement(self, elementType: ElementType) -> None: ...
    def PromptToPlaceElementTypeOnLegendView(self, elementType: ElementType) -> None: ...
    def PromptToMatchElementType(self, elementType: ElementType) -> None: ...
    def CanPlaceElementType(self, elementType: ElementType) -> bool: ...
    @overload
    def SaveAs(self, options: UISaveAsOptions) -> None: ...
    @overload
    def SaveAs(self) -> None: ...
    def GetPlacementTypes(self, familySymbol: FamilySymbol, pDBView: View) -> List[FaceBasedPlacementType]: ...
    def UpdateAllOpenViews(self) -> None: ...
    def Dispose(self) -> None: ...


class ExternalCommandData(APIObject):
    @property
    def Application(self) -> UIApplication: ...
    @Application.setter
    def Application(self, application: UIApplication) -> None: ...
    @property
    def View(self) -> View: ...
    @View.setter
    def View(self, view: View) -> None: ...
    @property
    def JournalData(self) -> IDictionary: ...
    @JournalData.setter
    def JournalData(self, data: IDictionary) -> None: ...


class DockablePaneState:
    @overload
    def __init__(self, other: DockablePaneState): ...
    @overload
    def __init__(self): ...
    @property
    def DockPosition(self) -> DockPosition: ...
    @DockPosition.setter
    def DockPosition(self, dockPosition: DockPosition) -> None: ...
    @property
    def FloatingRectangle(self) -> Rectangle: ...
    @property
    def TabBehind(self) -> DockablePaneId: ...
    @TabBehind.setter
    def TabBehind(self, tabBehind: DockablePaneId) -> None: ...
    @property
    def MinimumWidth(self) -> int: ...
    @MinimumWidth.setter
    def MinimumWidth(self, minimumWidth: int) -> None: ...
    @property
    def MinimumHeight(self) -> int: ...
    @MinimumHeight.setter
    def MinimumHeight(self, minimumHeight: int) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def SetFloatingRectangle(self, rect: Rectangle) -> None: ...
    def Dispose(self) -> None: ...


class DockablePane:
    @overload
    def __init__(self, other: DockablePane): ...
    @overload
    def __init__(self, id: DockablePaneId): ...
    @property
    def Id(self) -> DockablePaneId: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetTitle(self) -> str: ...
    def Show(self) -> None: ...
    def Hide(self) -> None: ...
    def IsShown(self) -> bool: ...
    def PaneIsBuiltIn(id: DockablePaneId) -> bool: ...
    def PaneIsRegistered(id: DockablePaneId) -> bool: ...
    def PaneExists(id: DockablePaneId) -> bool: ...
    def Dispose(self) -> None: ...


class IFrameworkElementCreator:
    def CreateFrameworkElement(self) -> FrameworkElement: ...


class EditorInteraction:
    @overload
    def __init__(self, interactionType: EditorInteractionType): ...
    @overload
    def __init__(self): ...
    @property
    def InteractionType(self) -> EditorInteractionType: ...
    @InteractionType.setter
    def InteractionType(self, interactionType: EditorInteractionType) -> None: ...


class DockablePaneProviderData:
    def __init__(self): ...
    def GetFrameworkElement(self) -> FrameworkElement: ...
    @property
    def FrameworkElement(self) -> FrameworkElement: ...
    @FrameworkElement.setter
    def FrameworkElement(self, elem: FrameworkElement) -> None: ...
    @property
    def FrameworkElementCreator(self) -> IFrameworkElementCreator: ...
    @FrameworkElementCreator.setter
    def FrameworkElementCreator(self, elemCreator: IFrameworkElementCreator) -> None: ...
    @property
    def InitialState(self) -> DockablePaneState: ...
    @InitialState.setter
    def InitialState(self, state: DockablePaneState) -> None: ...
    @property
    def ContextualHelp(self) -> ContextualHelp: ...
    @ContextualHelp.setter
    def ContextualHelp(self, help: ContextualHelp) -> None: ...
    @property
    def EditorInteraction(self) -> EditorInteraction: ...
    @EditorInteraction.setter
    def EditorInteraction(self, interaction: EditorInteraction) -> None: ...
    @property
    def VisibleByDefault(self) -> bool: ...
    @VisibleByDefault.setter
    def VisibleByDefault(self, visibleByDefault: bool) -> None: ...


class IDockablePaneProvider:
    def SetupDockablePane(self, data: DockablePaneProviderData) -> None: ...


class UIApplication:
    def __init__(self, revitApp: Application): ...
    @property
    def ActiveUIDocument(self) -> UIDocument: ...
    @property
    def MainWindowExtents(self) -> Rectangle: ...
    @property
    def DrawingAreaExtents(self) -> Rectangle: ...
    @overload
    def DoDragDrop(dropData: Object, handler: IDropHandler) -> None: ...
    @overload
    def DoDragDrop(dropData: ICollection) -> None: ...
    def CanPostCommand(self, commandId: RevitCommandId) -> bool: ...
    def PostCommand(self, commandId: RevitCommandId) -> None: ...
    def CreateRibbonTab(self, tabName: str) -> None: ...
    @overload
    def CreateRibbonPanel(self, tab: Tab, panelName: str) -> RibbonPanel: ...
    @overload
    def CreateRibbonPanel(self, panelName: str) -> RibbonPanel: ...
    @overload
    def CreateRibbonPanel(self, tabName: str, panelName: str) -> RibbonPanel: ...
    @overload
    def GetRibbonPanels(self, tabName: str) -> List: ...
    @overload
    def GetRibbonPanels(self, tab: Tab) -> List: ...
    @overload
    def GetRibbonPanels(self) -> List: ...
    @property
    def LoadedApplications(self) -> ExternalApplicationArray: ...
    @property
    def ActiveAddInId(self) -> AddInId: ...
    def CreateAddInCommandBinding(self, revitCommandId: RevitCommandId) -> AddInCommandBinding: ...
    def RemoveAddInCommandBinding(self, revitCommandId: RevitCommandId) -> None: ...
    def LoadAddIn(self, fileName: str) -> None: ...
    def LoadPackageContents(self, packageContentsPath: str) -> None: ...
    def GetDockablePane(self, id: DockablePaneId) -> DockablePane: ...
    @property
    def ProductIsRS(self) -> bool: ...
    def RegisterDockablePane(self, id: DockablePaneId, title: str, provider: IDockablePaneProvider) -> None: ...
    @property
    def MainWindowHandle(self) -> IntPtr: ...
    @property
    def Application(self) -> Application: ...
    @property
    def IsViewerModeActive(self) -> bool: ...
    @property
    def IsValidObject(self) -> bool: ...
    @overload
    def OpenAndActivateDocument(self, modelPath: ModelPath, openOptions: OpenOptions, detachAndPrompt: bool, openFromCloudCallback: IOpenFromCloudCallback) -> UIDocument: ...
    @overload
    def OpenAndActivateDocument(self, modelPath: ModelPath, openOptions: OpenOptions, detachAndPrompt: bool) -> UIDocument: ...
    @overload
    def OpenAndActivateDocument(self, fileName: str) -> UIDocument: ...
    def add_ViewActivating(self, handler: EventHandler) -> None: ...
    def remove_ViewActivating(self, handler: EventHandler) -> None: ...
    def add_ViewActivated(self, handler: EventHandler) -> None: ...
    def remove_ViewActivated(self, handler: EventHandler) -> None: ...
    def add_DialogBoxShowing(self, handler: EventHandler) -> None: ...
    def remove_DialogBoxShowing(self, handler: EventHandler) -> None: ...
    def add_FormulaEditing(self, handler: EventHandler) -> None: ...
    def remove_FormulaEditing(self, handler: EventHandler) -> None: ...
    def add_Idling(self, handler: EventHandler) -> None: ...
    def remove_Idling(self, handler: EventHandler) -> None: ...
    def add_ApplicationClosing(self, handler: EventHandler) -> None: ...
    def remove_ApplicationClosing(self, handler: EventHandler) -> None: ...
    def add_DisplayingOptionsDialog(self, handler: EventHandler) -> None: ...
    def remove_DisplayingOptionsDialog(self, handler: EventHandler) -> None: ...
    def add_DockableFrameVisibilityChanged(self, handler: EventHandler) -> None: ...
    def remove_DockableFrameVisibilityChanged(self, handler: EventHandler) -> None: ...
    def add_DockableFrameFocusChanged(self, handler: EventHandler) -> None: ...
    def remove_DockableFrameFocusChanged(self, handler: EventHandler) -> None: ...
    def add_FabricationPartBrowserChanged(self, handler: EventHandler) -> None: ...
    def remove_FabricationPartBrowserChanged(self, handler: EventHandler) -> None: ...
    def add_TransferringProjectStandards(self, handler: EventHandler) -> None: ...
    def remove_TransferringProjectStandards(self, handler: EventHandler) -> None: ...
    def add_TransferredProjectStandards(self, handler: EventHandler) -> None: ...
    def remove_TransferredProjectStandards(self, handler: EventHandler) -> None: ...
    def Dispose(self) -> None: ...


class Tab:
    AddIns = 0
    Analyze = 1


class FaceBasedPlacementType:
    Default = 0
    PlaceOnVerticalFace = 1
    PlaceOnFace = 2
    PlaceOnWorkPlane = 3


class SketchGalleryOptions:
    SGO_Default = 0
    SGO_Line = 1
    SGO_Rect = 2
    SGO_InscribedPolygon = 3
    SGO_CircumscribedPolygon = 4
    SGO_Circle = 5
    SGO_Arc3Point = 6
    SGO_ArcCenterEnds = 7
    SGO_ArcTanEnd = 8
    SGO_ArcFillet = 9
    SGO_Spline = 10
    SGO_SplineByPoints = 11
    SGO_FullEllipse = 12
    SGO_PartialEllipse = 13
    SGO_PickLines = 14
    SGO_PickFaces = 15
    SGO_PickWalls = 16
    SGO_PickSupports = 17
    SGO_PickRoofs = 18
    SGO_Point = 19
    SGO_PickPoints = 20
    SGO_PointElement = 21
    SGO_RunLine = 22
    SGO_RunArcFullStep = 23
    SGO_RunArcCenterEnds = 24
    SGO_LandingSquare = 25
    SGO_WinderLShape = 26
    SGO_WinderPattern = 27
    SGO_SupportPickLine = 28
    SGO_WinderUShape = 29
    SGO_LandingWithTwoRuns = 30
    SGO_SketchRun = 31
    SGO_SketchLanding = 32


class EditorInteractionType:
    Dismiss = 0
    KeepAlive = 1


class DockPosition:
    Top = 59419
    Left = 59420
    Right = 59421
    Bottom = 59422
    Floating = 59423
    Tabbed = 59424


class IExternalEventHandler:
    def Execute(self, app: UIApplication) -> None: ...
    def GetName(self) -> str: ...


class ExternalEvent:
    @property
    def IsPending(self) -> bool: ...
    def Create(handler: IExternalEventHandler) -> ExternalEvent: ...
    def CreateJournalable(handler: IExternalEventHandler) -> ExternalEvent: ...
    def Raise(self) -> ExternalEventRequest: ...
    def Dispose(self) -> None: ...


class ExternalEventRequest:
    Accepted = 0
    Pending = 1
    Denied = 2
    TimedOut = 3


class RevitCommandId:
    def LookupPostableCommandId(postableCommand: PostableCommand) -> RevitCommandId: ...
    @property
    def Name(self) -> str: ...
    @property
    def Id(self) -> UInt32: ...
    @property
    def CanHaveBinding(self) -> bool: ...
    @property
    def HasBinding(self) -> bool: ...
    @property
    def IsValidObject(self) -> bool: ...
    def LookupCommandId(name: str) -> RevitCommandId: ...
    def Dispose(self) -> None: ...


class PostableCommand:
    RoofByFace = 20
    Soffit = 493
    RepeatComponent = 840
    ViewList = 1075
    LinearDimensionTypes = 1081
    AngularDimensionTypes = 1082
    SplitElement = 1100
    SpotElevation = 1109
    SpotCoordinate = 1114
    RoofByFootprint = 1142
    RoofByExtrusion = 1143
    RadialDimensionTypes = 1199
    ReloadLatest = 1729
    Render = 1782
    ShowHistory = 1985
    SplitSurface = 1992
    DrawOnFace = 2706
    DrawOnWorkPlane = 2707
    GrayInactiveWorksets = 2765
    ImportGBXML = 3102
    ExportReportsRoomOrAreaReport = 3125
    SheetIssuesOrRevisions = 3153
    RevisionSchedule = 3154
    CalloutTags = 3167
    ElevationTags = 3168
    SectionTags = 3169
    FloorByFaceFloor = 3234
    HideCategory = 3240
    RunInterferenceCheck = 3259
    SpecifyCoordinatesAtPoint = 3266
    ExportIFC = 3278
    SystemBrowser = 3280
    PublishDWGToAutodeskBuzzsaw = 3283
    PublishDWFToAutodeskBuzzsaw = 3284
    PublishDXFToAutodeskBuzzsaw = 3285
    PublishDGNToAutodeskBuzzsaw = 3287
    ResetAnalyticalModel = 3288
    PublishSATToAutodeskBuzzsaw = 3289
    ExportOptionsIFCOptions = 3292
    TypeProperties = 3302
    ExportDWFOrDWFx = 3341
    ExportCADFormatsDWG = 3342
    ExportCADFormatsDXF = 3343
    ExportCADFormatsDGN = 3344
    ExportCADFormatsACIS_SAT = 3345
    ExportCADFormatsSTL = 3346
    RelinquishAllMine = 3363
    MaterialTakeoff = 3388
    DWFMarkup = 3401
    ImportTypes = 3404
    ExportExportTypes = 3405
    CheckMemberSupports = 3410
    AnalyticalConsistencyChecks = 3411
    KeynoteLegend = 3449
    KeynotingSettings = 3450
    OpenBuildingComponent = 3470
    Web = 3619
    TopChord = 3620
    BottomChord = 3621
    TogglePropertiesPalette = 4534
    StatusBarWorksets = 4609
    StatusBarDesignOptions = 4610
    ReinforcementNumbers = 4700
    FamilyCategoryAndParameters = 10133
    ExportImagesandAnimationsWalkthrough = 12059
    ExportImagesandAnimationsSolarStudy = 12075
    SunSettings = 12299
    Grid = 32770
    ArchitecturalWall = 32771
    Door = 32772
    Window = 32773
    Delete = 32778
    Space = 32784
    SpaceTag = 32785
    SpaceSeparator = 32786
    Level = 32794
    FramingElevation = 32814
    ArchitecturalFloor = 32914
    Stair = 32916
    ArchitecturalColumn = 32918
    MirrorProject = 32919
    RotateProjectNorth = 32920
    Rotate = 32934
    MirrorPickAxis = 32936
    LineWeights = 32946
    ObjectStyles = 32947
    AnalysisDisplayStyles = 32948
    Snaps = 32949
    ProjectUnits = 32950
    StructuralPlan = 32952
    FloorPlan = 32953
    Section = 32955
    Camera = 32956
    ProjectBrowser = 32957
    NewSheet = 32958
    ImportCAD = 32959
    LinkCAD = 32961
    PointCloud = 32963
    NewFamilyFile = 32983
    LoadShapes = 32984
    NewConceptualMass = 32986
    Pin = 32997
    Unpin = 33001
    SolidExtrusion = 33002
    PlaceAComponent = 33003
    ModelLine = 33006
    StairPath = 33010
    ReferencePlane = 33026
    Redo = 33031
    Undo = 33032
    Control = 33034
    Filters = 33099
    TemporaryDimensions = 33100
    LinePatterns = 33120
    Array = 33121
    Move = 33127
    Copy = 33129
    PlaceView = 33132
    Text = 33134
    NewTitleBlock = 33140
    AutomaticCeiling = 33158
    FillPatterns = 33163
    Default3DView = 33169
    Label = 33174
    VisibilityOrGraphics = 33175
    ViewRange = 33176
    Opening = 33177
    NewRevitFile = 33179
    ActivateView = 33181
    Materials = 33184
    DeactivateView = 33185
    MaterialAssets = 33186
    DecalTypes = 33188
    Options = 33195
    Room = 33198
    CurtainGrid = 33199
    CurtainWallMullion = 33200
    FamilyTypes = 33203
    FilledRegion = 33204
    RoomTag = 33208
    Align = 33218
    Symbol = 33221
    DuplicateView = 33245
    ExportOptionsExportSetupsDGN = 33248
    ExportOptionsExportSetupsDWGOrDXF = 33251
    ColorFillLegend = 33252
    BuildingElevation = 33255
    JoinOrUnjoinRoof = 33277
    SolidBlend = 33278
    ManageLinks = 33279
    SolidRevolve = 33287
    ScheduleOrQuantities = 33296
    Callout = 33298
    SolidSweep = 33300
    CreateGroup = 33305
    Ramp = 33316
    RoomSeparator = 33318
    AlignedToCurrentView = 33320
    AlignedToPickedLevel = 33321
    WallJoins = 33323
    StartingView = 33327
    AlignedToSelectedLevels = 33328
    TagByCategory = 33329
    AlignedToSelectedViews = 33332
    LoadedTagsAndSymbols = 33338
    NewAnnotationSymbol = 33339
    AlignedToSamePlace = 33341
    SpotSlopeTypes = 33348
    Arrowheads = 33349
    SpotElevationTypes = 33350
    SpotCoordinateTypes = 33351
    ReviewWarnings = 33352
    LineStyles = 33360
    ReflectedCeilingPlan = 33380
    DetailLevel = 33384
    VoidExtrusion = 33397
    VoidBlend = 33398
    VoidRevolve = 33399
    VoidSweep = 33400
    CutGeometry = 33407
    UncutGeometry = 33408
    PlaceDetailGroup = 33421
    OverrideByCategory = 33422
    DetailComponent = 33424
    DetailLine = 33425
    Insulation = 33432
    CreateSimilar = 33441
    Worksets = 33460
    Phases = 33472
    PropertyLine = 33480
    BuildingPad = 33481
    MatchTypeProperties = 33513
    ScopeBox = 33519
    Linework = 33520
    SetWorkPlane = 33527
    DraftingView = 33530
    Legend = 33531
    Demolish = 33534
    RevisionCloud = 33540
    SynchronizeAndModifySettings = 33542
    ProjectInformation = 33543
    ModelInPlace = 33544
    SheetList = 33631
    AreaPlan = 33638
    GradedRegion = 33640
    Toposurface = 33641
    ManageImages = 33648
    ApplyTemplatePropertiesToCurrentView = 33651
    RevealWall = 33654
    SweepWall = 33655
    Paint = 33656
    RemovePaint = 33657
    NoteBlock = 33659
    ModelText = 33660
    ShowHiddenLinesByElement = 33671
    RemoveHiddenLinesByElement = 33673
    ManageViewTemplates = 33683
    CreateTemplateFromCurrentView = 33684
    RestoreBackup = 33690
    ExportODBCDatabase = 33693
    LoadAsGroup = 33695
    SaveAsLibraryGroup = 33696
    ParkingComponent = 33698
    TransferProjectStandards = 33699
    SymbolicLine = 33700
    CutProfile = 33709
    ExportReportsSchedule = 33710
    CloseInactiveViews = 33712
    AreaTag = 33718
    AreaBoundary = 33719
    Area = 33720
    JoinGeometry = 33729
    UnjoinGeometry = 33730
    SwitchJoinOrder = 33731
    TagAllNotTagged = 33735
    MergeSurfaces = 33742
    SharedParameters = 33748
    LegendComponent = 33751
    Offset = 33757
    PurgeUnused = 33780
    PlaceDecal = 33789
    RebarLine = 33810
    LabelContours = 33820
    SplitWithGap = 33829
    ThinLines = 33834
    LinkRevit = 33836
    ExportImagesandAnimationsImage = 33838
    ProjectParameters = 33855
    RelocateProject = 33856
    RotateTrueNorth = 33857
    DuplicateWithDetailing = 33858
    Location = 33862
    AcquireCoordinates = 33863
    ReportSharedCoordinates = 33865
    IdsOfSelection = 33866
    SelectById = 33867
    PublishCoordinates = 33869
    PlanRegion = 33900
    BrowserOrganization = 33904
    Matchline = 33905
    ViewReference = 33906
    DesignOptions = 33907
    AddToSet = 33908
    PickToEdit = 33910
    EditingRequests = 33911
    SaveAsLibraryFamily = 33923
    Subregion = 34964
    Railing = 34969
    Beam = 34972
    Brace = 34973
    StructuralColumn = 34974
    StructuralWall = 34975
    SpanDirectionSymbol = 34980
    StructuralFloor = 34985
    Scale = 34986
    Loads = 34987
    StructuralSettings = 34993
    ReferenceLine = 34994
    RebarCoverSettings = 35003
    ShowMassFormAndFloors = 35060
    CurtainSystemByFace = 35072
    WallByFaceWall = 35074
    ShowMassByViewSettings = 35075
    ShowMassSurfaceTypes = 35078
    ShowMassZonesAndShades = 35079
    StructuralRebar = 35098
    UseCurrentProject = 35110
    SelectLink = 35111
    Wall = 35113
    Isolated = 35114
    MaterialTag = 35145
    CoordinationSettings = 35149
    Slab = 35156
    ElementKeynote = 35158
    MaterialKeynote = 35159
    UserKeynote = 35160
    LoadSelection = 35180
    SaveSelection = 35181
    EditSelection = 35183
    MultiCategoryTag = 35203
    BoundaryConditions = 35223
    StructuralPathReinforcement = 35225
    PathReinforcementSymbol = 35228
    OpeningByFace = 35253
    VerticalOpening = 35254
    DormerOpening = 35255
    WallOpening = 35256
    ShaftOpening = 35257
    SpotSlope = 35259
    HideElements = 35261
    OverrideByElement = 35265
    OverrideByFilter = 35266
    DuplicateAsDependent = 35280
    MaskingRegion = 35287
    MajorSegment = 35293
    ReinforcementSettings = 35300
    ExportFBX = 35350
    RecentFiles = 35364
    OpenFamilyFile = 35365
    CreateParts = 35425
    ViewCube = 35458
    BeamAnnotations = 35585
    AllowableBarTypes = 35599
    MacroManager = 35602
    MacroSecurity = 35603
    NavigationBar = 35631
    ExportBuildingSite = 35674
    FormWorkPlaneView = 35703
    ReconcileHosting = 35708
    SplitFace = 35732
    AdjustAnalyticalModel = 35739
    DiameterDimension = 35778
    DiameterDimensionTypes = 35779
    StructuralFabricArea = 35781
    FabricReinforcementSymbol = 35793
    PlaceOnHost = 35807
    DuctPressureLossReport = 35811
    PipePressureLossReport = 35812
    OpenSampleFiles = 35815
    SingleFabricSheetPlacement = 35823
    AlignedMultiRebarAnnotation = 35831
    LinearMultiRebarAnnotation = 35832
    FabricationSettings = 35839
    LinkIFC = 35853
    SelectionBox = 35857
    DuctAccessory = 37004
    AirTerminal = 37005
    Duct = 37006
    MechanicalEquipment = 37008
    DuctFitting = 37009
    FlexDuct = 37010
    ConvertToFlexDuct = 37014
    DuctLegend = 37027
    PanelSchedules = 37053
    ElectricalSettings = 37054
    LoadClassifications = 37055
    LightingFixture = 37059
    ElectricalEquipment = 37060
    ElectricalFixture = 37061
    CheckCircuits = 37062
    DemandFactors = 37068
    Pipe = 37100
    FlexPipe = 37102
    PipeFitting = 37103
    PipeAccessory = 37105
    ArcWire = 37106
    PipeLegend = 37149
    CheckPipeSystems = 37155
    CheckDuctSystems = 37167
    MechanicalSettings = 37168
    PlumbingFixture = 37203
    HeatingAndCoolingLoads = 37204
    Sprinkler = 37206
    ElectricalConnector = 37217
    DuctConnector = 37218
    PipeConnector = 37219
    Zone = 37244
    BuildingOrSpaceTypeSettings = 37259
    ChamferedWire = 37260
    SplineWire = 37261
    Communication = 37278
    Data = 37279
    FireAlarm = 37280
    Lighting = 37281
    NurseCall = 37282
    Security = 37283
    Telephone = 37284
    CableTrayConnector = 37291
    ConduitConnector = 37292
    CableTrayFitting = 37295
    ConduitFitting = 37296
    CableTray = 37297
    Conduit = 37298
    ManageTemplates = 37314
    EditATemplate = 37315
    ParallelConduits = 37361
    DuctPlaceholder = 37427
    PipePlaceholder = 37428
    ParallelPipes = 37441
    ShowDisconnects = 37444
    SiteComponent = 40239
    AreaAndVolumeComputations = 40382
    ShowWorkPlane = 40484
    Fascia = 40601
    Gutter = 40602
    SlabEdgeFloor = 40603
    CheckSpelling = 40986
    RepeatingDetailComponent = 42058
    InsertViewsFromFile = 42309
    Insert2DElementsFromFile = 42310
    SaveAsLibraryView = 42312
    AutomaticBeamSystem = 42398
    PlaceMass = 42644
    InPlaceMass = 42645
    SynchronizeNow = 42654
    ExportGBXML = 42672
    StructuralTrusses = 43213
    ShowLastReport = 46061
    GraphicalColumnSchedule = 46139
    ApplyCoping = 46450
    RemoveCoping = 46451
    BeamSystemSymbol = 46567
    StructuralAreaReinforcement = 46932
    AreaReinforcementSymbol = 46959
    SolidSweptBlend = 47163
    VoidSweptBlend = 47164
    EditRebarCover = 48583
    ColorSchemes = 48739
    TrimOrExtendToCorner = 48952
    TrimOrExtendSingleElement = 48953
    TrimOrExtendMultipleElements = 48954
    AlignedDimension = 48955
    LinearDimension = 48956
    AngularDimension = 48957
    RadialDimension = 48958
    ArcLengthDimension = 48959
    MeasureBetweenTwoReferences = 48962
    MeasureAlongAnElement = 48963
    MirrorDrawAxis = 49000
    KeyboardShortcuts = 49211
    BeamOrColumnJoins = 49543
    SaveAsTemplate = 49783
    HalftoneOrUnderlay = 49786
    StairBySketch = 50126
    LoadCases = 53531
    LoadCombinations = 53532
    FindOrReplace = 53591
    EnergySettings = 53612
    RunEnergySimulation = 53613
    ResultsAndCompare = 53614
    ExportMassModelGBXML = 53615
    GuideGrid = 53675
    CreateAssembly = 53677
    StairTreadOrRiserNumber = 53827
    ManageConnectionToARevitServerAccelerator = 53949
    RenderInCloud = 53961
    RenderGallery = 53962
    ShowEnergyModel = 53964
    DisplaceElements = 54029
    AssemblyCode = 54096
    FabricationPart = 54149
    Playlist = 57282
    OpenRevitFile = 57601
    Close = 57602
    Save = 57603
    SaveAsProject = 57604
    PrintSetup = 57606
    Print = 57607
    PrintPreview = 57609
    ModelBrowserContainer = 57612
    CopyToClipboard = 57634
    CutToClipboard = 57635
    PasteFromClipboard = 57637
    TabViews = 57650
    TileViews = 57652
    ExitRevit = 57665
    StatusBar = 59393


class UIControlledApplication:
    @property
    def ControlledApplication(self) -> ControlledApplication: ...
    @property
    def IsLateAddinLoading(self) -> bool: ...
    @property
    def IsViewerModeActive(self) -> bool: ...
    def CreateRibbonTab(self, tabName: str) -> None: ...
    @overload
    def CreateRibbonPanel(self, tab: Tab, panelName: str) -> RibbonPanel: ...
    @overload
    def CreateRibbonPanel(self, panelName: str) -> RibbonPanel: ...
    @overload
    def CreateRibbonPanel(self, tabName: str, panelName: str) -> RibbonPanel: ...
    @overload
    def GetRibbonPanels(self, tabName: str) -> List: ...
    @overload
    def GetRibbonPanels(self, tab: Tab) -> List: ...
    @overload
    def GetRibbonPanels(self) -> List: ...
    @property
    def LoadedApplications(self) -> ExternalApplicationArray: ...
    @property
    def ActiveAddInId(self) -> AddInId: ...
    def CreateAddInCommandBinding(self, revitCommandId: RevitCommandId) -> AddInCommandBinding: ...
    def RemoveAddInCommandBinding(self, revitCommandId: RevitCommandId) -> None: ...
    def LoadAddIn(self, fileName: str) -> None: ...
    def LoadPackageContents(self, packageContentsPath: str) -> None: ...
    def GetDockablePane(self, id: DockablePaneId) -> DockablePane: ...
    @property
    def ProductIsRS(self) -> bool: ...
    def RegisterDockablePane(self, id: DockablePaneId, title: str, provider: IDockablePaneProvider) -> None: ...
    @property
    def MainWindowHandle(self) -> IntPtr: ...
    def add_ViewActivating(self, handler: EventHandler) -> None: ...
    def remove_ViewActivating(self, handler: EventHandler) -> None: ...
    def add_ViewActivated(self, handler: EventHandler) -> None: ...
    def remove_ViewActivated(self, handler: EventHandler) -> None: ...
    def add_FormulaEditing(self, handler: EventHandler) -> None: ...
    def remove_FormulaEditing(self, handler: EventHandler) -> None: ...
    def add_ApplicationClosing(self, handler: EventHandler) -> None: ...
    def remove_ApplicationClosing(self, handler: EventHandler) -> None: ...
    def add_DialogBoxShowing(self, handler: EventHandler) -> None: ...
    def remove_DialogBoxShowing(self, handler: EventHandler) -> None: ...
    def add_Idling(self, handler: EventHandler) -> None: ...
    def remove_Idling(self, handler: EventHandler) -> None: ...
    def add_DisplayingOptionsDialog(self, handler: EventHandler) -> None: ...
    def remove_DisplayingOptionsDialog(self, handler: EventHandler) -> None: ...
    def add_DockableFrameVisibilityChanged(self, handler: EventHandler) -> None: ...
    def remove_DockableFrameVisibilityChanged(self, handler: EventHandler) -> None: ...
    def add_DockableFrameFocusChanged(self, handler: EventHandler) -> None: ...
    def remove_DockableFrameFocusChanged(self, handler: EventHandler) -> None: ...
    def add_FabricationPartBrowserChanged(self, handler: EventHandler) -> None: ...
    def remove_FabricationPartBrowserChanged(self, handler: EventHandler) -> None: ...
    def add_TransferringProjectStandards(self, handler: EventHandler) -> None: ...
    def remove_TransferringProjectStandards(self, handler: EventHandler) -> None: ...
    def add_TransferredProjectStandards(self, handler: EventHandler) -> None: ...
    def remove_TransferredProjectStandards(self, handler: EventHandler) -> None: ...


class IExternalApplication:
    def OnStartup(self, application: UIControlledApplication) -> Result: ...
    def OnShutdown(self, application: UIControlledApplication) -> Result: ...


class ExternalApplicationArray(APIObject):
    def __init__(self): ...
    def Clear(self) -> None: ...
    @property
    def Size(self) -> int: ...
    @property
    def IsEmpty(self) -> bool: ...
    def ForwardIterator(self) -> ExternalApplicationArrayIterator: ...
    def ReverseIterator(self) -> ExternalApplicationArrayIterator: ...
    def GetEnumerator(self) -> IEnumerator: ...
    @property
    def Item(self, index: int) -> IExternalApplication: ...
    @Item.setter
    def Item(self, index: int, item: IExternalApplication) -> None: ...
    def Append(self, item: IExternalApplication) -> None: ...
    def Insert(self, item: IExternalApplication, index: int) -> None: ...


class ExternalApplicationArrayIterator(APIObject):
    def __init__(self): ...
    @property
    def Current(self) -> Object: ...
    def MoveNext(self) -> bool: ...
    def Reset(self) -> None: ...


class AddInCommandBinding:
    @property
    def RevitCommandId(self) -> RevitCommandId: ...
    def add_BeforeExecuted(self, handler: EventHandler) -> None: ...
    def remove_BeforeExecuted(self, handler: EventHandler) -> None: ...
    def add_Executed(self, handler: EventHandler) -> None: ...
    def remove_Executed(self, handler: EventHandler) -> None: ...
    def add_CanExecute(self, handler: EventHandler) -> None: ...
    def remove_CanExecute(self, handler: EventHandler) -> None: ...


class Result:
    Succeeded = 0
    Cancelled = 1
    Failed = -1


class PromptForFamilyInstancePlacementOptions:
    @overload
    def __init__(self, other: PromptForFamilyInstancePlacementOptions): ...
    @overload
    def __init__(self): ...
    @property
    def PlaceAirTerminalOnDuct(self) -> bool: ...
    @PlaceAirTerminalOnDuct.setter
    def PlaceAirTerminalOnDuct(self, placeAirTerminalOnDuct: bool) -> None: ...
    @property
    def SketchGalleryOptions(self) -> SketchGalleryOptions: ...
    @SketchGalleryOptions.setter
    def SketchGalleryOptions(self, sketchGalleryOptions: SketchGalleryOptions) -> None: ...
    @property
    def FaceBasedPlacementType(self) -> FaceBasedPlacementType: ...
    @FaceBasedPlacementType.setter
    def FaceBasedPlacementType(self, faceBasedPlacementType: FaceBasedPlacementType) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class UIView:
    @property
    def ViewId(self) -> ElementId: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetWindowRectangle(self) -> Rectangle: ...
    def Close(self) -> None: ...
    def ZoomToFit(self) -> None: ...
    def Zoom(self, zoomFactor: float) -> None: ...
    def ZoomSheetSize(self) -> None: ...
    def ZoomAndCenterRectangle(self, viewCorner1: XYZ, viewCorner2: XYZ) -> None: ...
    def GetZoomCorners(self) -> List[XYZ]: ...
    def Dispose(self) -> None: ...


class ContextualHelp:
    def __init__(self, helpType: ContextualHelpType, helpPath: str): ...
    @property
    def HelpType(self) -> ContextualHelpType: ...
    @HelpType.setter
    def HelpType(self, helpType: ContextualHelpType) -> None: ...
    @property
    def HelpPath(self) -> str: ...
    @HelpPath.setter
    def HelpPath(self, helpPath: str) -> None: ...
    @property
    def HelpTopicUrl(self) -> str: ...
    @HelpTopicUrl.setter
    def HelpTopicUrl(self, helpPath: str) -> None: ...
    def Launch(self) -> None: ...


class RibbonItem:
    @property
    def Name(self) -> str: ...
    @property
    def ItemText(self) -> str: ...
    @ItemText.setter
    def ItemText(self, text: str) -> None: ...
    @property
    def ToolTip(self) -> str: ...
    @ToolTip.setter
    def ToolTip(self, toolTip: str) -> None: ...
    @property
    def LongDescription(self) -> str: ...
    @LongDescription.setter
    def LongDescription(self, longDescription: str) -> None: ...
    @property
    def ToolTipImage(self) -> ImageSource: ...
    @ToolTipImage.setter
    def ToolTipImage(self, image: ImageSource) -> None: ...
    @property
    def Enabled(self) -> bool: ...
    @Enabled.setter
    def Enabled(self, bEnabled: bool) -> None: ...
    @property
    def Visible(self) -> bool: ...
    @Visible.setter
    def Visible(self, bVisible: bool) -> None: ...
    @property
    def ItemType(self) -> RibbonItemType: ...
    def Equals(self, obj: Object) -> bool: ...
    def GetContextualHelp(self) -> ContextualHelp: ...
    def SetContextualHelp(self, contextualHelp: ContextualHelp) -> None: ...


class RibbonItemData:
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, name: str) -> None: ...
    @property
    def ToolTip(self) -> str: ...
    @ToolTip.setter
    def ToolTip(self, toolTip: str) -> None: ...
    @property
    def LongDescription(self) -> str: ...
    @LongDescription.setter
    def LongDescription(self, longDescription: str) -> None: ...
    @property
    def ToolTipImage(self) -> ImageSource: ...
    @ToolTipImage.setter
    def ToolTipImage(self, image: ImageSource) -> None: ...
    def GetContextualHelp(self) -> ContextualHelp: ...
    def SetContextualHelp(self, contextualHelp: ContextualHelp) -> None: ...


class RibbonItemType:
    PushButton = 0
    PulldownButton = 1
    SplitButton = 2
    ToggleButton = 3
    RadioButtonGroup = 4
    ComboBoxMember = 5
    ComboBox = 6
    TextBox = 7


class ContextualHelpType:
    #None = 0
    ContextId = 1
    Url = 2
    ChmFile = 3


class DockablePaneId(GuidEnum):
    def __init__(self, guid: Guid): ...
    def op_Equality(first: DockablePaneId, second: DockablePaneId) -> bool: ...
    def op_Inequality(first: DockablePaneId, second: DockablePaneId) -> bool: ...


class PreviewControl:
    def __init__(self, document: Document, viewId: ElementId): ...
    @property
    def ViewId(self) -> ElementId: ...
    @property
    def UIView(self) -> UIView: ...
    @property
    def ScrollbarVisibility(self) -> ScrollbarVisibility: ...
    @ScrollbarVisibility.setter
    def ScrollbarVisibility(self, scrollbarVisibility: ScrollbarVisibility) -> None: ...
    def Dispose(self) -> None: ...


class ScrollbarVisibility:
    #None = 0
    Horizontal = 1
    Vertical = 2
    Both = 3


class IExternalCommand:
    def Execute(self, commandData: ExternalCommandData) -> Tuple[Result, str, ElementSet]: ...


class IExternalCommandAvailability:
    def IsCommandAvailable(self, applicationData: UIApplication, selectedCategories: CategorySet) -> bool: ...


class ComboBoxMember(RibbonItem):
    @property
    def Image(self) -> ImageSource: ...
    @Image.setter
    def Image(self, value: ImageSource) -> None: ...
    @property
    def GroupName(self) -> str: ...


class ComboBoxMemberData(RibbonItemData):
    def __init__(self, name: str, text: str): ...
    @property
    def Text(self) -> str: ...
    @Text.setter
    def Text(self, text: str) -> None: ...
    @property
    def Image(self) -> ImageSource: ...
    @Image.setter
    def Image(self, value: ImageSource) -> None: ...
    @property
    def GroupName(self) -> str: ...
    @GroupName.setter
    def GroupName(self, value: str) -> None: ...


class ComboBox(RibbonItem):
    @property
    def Current(self) -> ComboBoxMember: ...
    @Current.setter
    def Current(self, value: ComboBoxMember) -> None: ...
    @property
    def Image(self) -> ImageSource: ...
    @Image.setter
    def Image(self, image: ImageSource) -> None: ...
    def AddItem(self, memberData: ComboBoxMemberData) -> ComboBoxMember: ...
    def AddItems(self, memberData: List[ComboBoxMemberData]) -> List[ComboBoxMember]: ...
    def AddSeparator(self) -> None: ...
    def GetItems(self) -> List[ComboBoxMember]: ...
    def add_DropDownOpened(self, value: EventHandler) -> None: ...
    def remove_DropDownOpened(self, value: EventHandler) -> None: ...
    def add_DropDownClosed(self, value: EventHandler) -> None: ...
    def remove_DropDownClosed(self, value: EventHandler) -> None: ...
    def add_CurrentChanged(self, value: EventHandler) -> None: ...
    def remove_CurrentChanged(self, value: EventHandler) -> None: ...


class ComboBoxData(RibbonItemData):
    def __init__(self, name: str): ...
    @property
    def Image(self) -> ImageSource: ...
    @Image.setter
    def Image(self, image: ImageSource) -> None: ...


class RadioButtonGroup(RibbonItem):
    @property
    def Current(self) -> ToggleButton: ...
    @Current.setter
    def Current(self, value: ToggleButton) -> None: ...
    def AddItem(self, buttonData: ToggleButtonData) -> ToggleButton: ...
    def AddItems(self, buttonData: List[ToggleButtonData]) -> List[ToggleButton]: ...
    def GetItems(self) -> List[ToggleButton]: ...


class RadioButtonGroupData(RibbonItemData):
    def __init__(self, name: str): ...


class RibbonButton(RibbonItem):
    @property
    def LargeImage(self) -> ImageSource: ...
    @LargeImage.setter
    def LargeImage(self, largeImage: ImageSource) -> None: ...
    @property
    def Image(self) -> ImageSource: ...
    @Image.setter
    def Image(self, image: ImageSource) -> None: ...
    @property
    def IsEnabledByContext(self) -> bool: ...


class PushButton(RibbonButton):
    @property
    def AssemblyName(self) -> str: ...
    @AssemblyName.setter
    def AssemblyName(self, assemblyName: str) -> None: ...
    @property
    def ClassName(self) -> str: ...
    @ClassName.setter
    def ClassName(self, className: str) -> None: ...
    @property
    def AvailabilityClassName(self) -> str: ...
    @AvailabilityClassName.setter
    def AvailabilityClassName(self, availabilityClassName: str) -> None: ...


class ButtonData(RibbonItemData):
    @property
    def Text(self) -> str: ...
    @Text.setter
    def Text(self, text: str) -> None: ...
    @property
    def LargeImage(self) -> ImageSource: ...
    @LargeImage.setter
    def LargeImage(self, largeImage: ImageSource) -> None: ...
    @property
    def Image(self) -> ImageSource: ...
    @Image.setter
    def Image(self, image: ImageSource) -> None: ...


class PushButtonData(ButtonData):
    def __init__(self, name: str, text: str, assemblyName: str, className: str): ...
    @property
    def ClassName(self) -> str: ...
    @ClassName.setter
    def ClassName(self, className: str) -> None: ...
    @property
    def AssemblyName(self) -> str: ...
    @AssemblyName.setter
    def AssemblyName(self, assemblyName: str) -> None: ...
    @property
    def AvailabilityClassName(self) -> str: ...
    @AvailabilityClassName.setter
    def AvailabilityClassName(self, availabilityClassName: str) -> None: ...


class PulldownButton(RibbonButton):
    def GetItems(self) -> List[PushButton]: ...
    def AddPushButton(self, buttonData: PushButtonData) -> PushButton: ...
    def AddSeparator(self) -> None: ...


class PulldownButtonData(ButtonData):
    def __init__(self, name: str, text: str): ...


class SplitButton(PulldownButton):
    @property
    def IsSynchronizedWithCurrentItem(self) -> bool: ...
    @IsSynchronizedWithCurrentItem.setter
    def IsSynchronizedWithCurrentItem(self, value: bool) -> None: ...
    @property
    def CurrentButton(self) -> PushButton: ...
    @CurrentButton.setter
    def CurrentButton(self, value: PushButton) -> None: ...


class SplitButtonData(PulldownButtonData):
    def __init__(self, name: str, text: str): ...


class TextBox(RibbonItem):
    @property
    def PromptText(self) -> str: ...
    @PromptText.setter
    def PromptText(self, value: str) -> None: ...
    @property
    def SelectTextOnFocus(self) -> bool: ...
    @SelectTextOnFocus.setter
    def SelectTextOnFocus(self, value: bool) -> None: ...
    @property
    def Value(self) -> Object: ...
    @Value.setter
    def Value(self, value: Object) -> None: ...
    @property
    def Image(self) -> ImageSource: ...
    @Image.setter
    def Image(self, image: ImageSource) -> None: ...
    @property
    def ShowImageAsButton(self) -> bool: ...
    @ShowImageAsButton.setter
    def ShowImageAsButton(self, value: bool) -> None: ...
    @property
    def Width(self) -> float: ...
    @Width.setter
    def Width(self, value: float) -> None: ...
    def add_EnterPressed(self, value: EventHandler) -> None: ...
    def remove_EnterPressed(self, value: EventHandler) -> None: ...


class TextBoxData(RibbonItemData):
    def __init__(self, name: str): ...
    @property
    def Image(self) -> ImageSource: ...
    @Image.setter
    def Image(self, image: ImageSource) -> None: ...


class ToggleButton(PushButton):


class ToggleButtonData(PushButtonData):
    @overload
    def __init__(self, name: str, text: str): ...
    @overload
    def __init__(self, name: str, text: str, assemblyName: str, className: str): ...


class TaskDialog(APIObject):
    def __init__(self, title: str): ...
    @property
    def Title(self) -> str: ...
    @Title.setter
    def Title(self, title: str) -> None: ...
    @property
    def MainInstruction(self) -> str: ...
    @MainInstruction.setter
    def MainInstruction(self, MainInstruction: str) -> None: ...
    @property
    def Id(self) -> str: ...
    @Id.setter
    def Id(self, Id: str) -> None: ...
    @property
    def MainContent(self) -> str: ...
    @MainContent.setter
    def MainContent(self, mainContent: str) -> None: ...
    @property
    def ExpandedContent(self) -> str: ...
    @ExpandedContent.setter
    def ExpandedContent(self, expandedContent: str) -> None: ...
    @property
    def VerificationText(self) -> str: ...
    @VerificationText.setter
    def VerificationText(self, verificationText: str) -> None: ...
    @property
    def ExtraCheckBoxText(self) -> str: ...
    @ExtraCheckBoxText.setter
    def ExtraCheckBoxText(self, extraText: str) -> None: ...
    @property
    def FooterText(self) -> str: ...
    @FooterText.setter
    def FooterText(self, footerText: str) -> None: ...
    @property
    def AllowCancellation(self) -> bool: ...
    @AllowCancellation.setter
    def AllowCancellation(self, allowCancellation: bool) -> None: ...
    @property
    def DefaultButton(self) -> TaskDialogResult: ...
    @DefaultButton.setter
    def DefaultButton(self, defaultButton: TaskDialogResult) -> None: ...
    @property
    def MainIcon(self) -> TaskDialogIcon: ...
    @MainIcon.setter
    def MainIcon(self, icon: TaskDialogIcon) -> None: ...
    @property
    def CommonButtons(self) -> TaskDialogCommonButtons: ...
    @CommonButtons.setter
    def CommonButtons(self, buttons: TaskDialogCommonButtons) -> None: ...
    @property
    def TitleAutoPrefix(self) -> bool: ...
    @TitleAutoPrefix.setter
    def TitleAutoPrefix(self, autoPrefix: bool) -> None: ...
    @property
    def EnableMarqueeProgressBar(self) -> bool: ...
    @EnableMarqueeProgressBar.setter
    def EnableMarqueeProgressBar(self, enableMarqueeProgressBar: bool) -> None: ...
    @overload
    def AddCommandLink(self, id: TaskDialogCommandLinkId, mainContent: str) -> None: ...
    @overload
    def AddCommandLink(self, id: TaskDialogCommandLinkId, mainContent: str, supportingContent: str) -> None: ...
    @overload
    def Show(title: str, mainInstruction: str, buttons: TaskDialogCommonButtons, defaultButton: TaskDialogResult) -> TaskDialogResult: ...
    @overload
    def Show(title: str, mainInstruction: str, buttons: TaskDialogCommonButtons) -> TaskDialogResult: ...
    @overload
    def Show(title: str, mainInstruction: str) -> TaskDialogResult: ...
    @overload
    def Show(self) -> TaskDialogResult: ...
    def WasVerificationChecked(self) -> bool: ...
    def WasExtraCheckBoxChecked(self) -> bool: ...
    def EnableDoNotShowAgain(self, dialogId: str, enableDoNotShow: bool, doNotShowText: str) -> None: ...


class TaskDialogCommonButtons:
    #None = 0
    Ok = 1
    Yes = 2
    No = 4
    Cancel = 8
    Retry = 16
    Close = 32


class TaskDialogCommandLinkId:
    CommandLink1 = 1001
    CommandLink2 = 1002
    CommandLink3 = 1003
    CommandLink4 = 1004


class TaskDialogResult:
    #None = 0
    Ok = 1
    Cancel = 2
    Retry = 4
    Yes = 6
    No = 7
    Close = 8
    CommandLink1 = 1001
    CommandLink2 = 1002
    CommandLink3 = 1003
    CommandLink4 = 1004


class TaskDialogIcon:
    TaskDialogIconNone = 0
    TaskDialogIconShield = 65532
    TaskDialogIconInformation = 65533
    TaskDialogIconError = 65534
    TaskDialogIconWarning = 65535


class UIThemeManager:
    def GetThemeName(frameTheme: UITheme) -> str: ...
    @property
    def DefaultTheme() -> UITheme: ...
    @DefaultTheme.setter
    def DefaultTheme(frameTheme: UITheme) -> None: ...
    @property
    def CurrentTheme() -> UITheme: ...
    @CurrentTheme.setter
    def CurrentTheme(frameTheme: UITheme) -> None: ...


class ColorSelectionDialog:
    def __init__(self): ...
    @property
    def SelectedColor(self) -> Color: ...
    @property
    def OriginalColor(self) -> Color: ...
    @OriginalColor.setter
    def OriginalColor(self, originalColor: Color) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Show(self) -> ItemSelectionDialogResult: ...
    def Dispose(self) -> None: ...


class DockablePanes:


class BuiltInDockablePanes:
    @property
    def ContentDelivery() -> DockablePaneId: ...
    @property
    def ElementView() -> DockablePaneId: ...
    @property
    def FabricationPartBrowser() -> DockablePaneId: ...
    @property
    def HostByLinkNavigator() -> DockablePaneId: ...
    @property
    def PnIdModelerBrowser() -> DockablePaneId: ...
    @property
    def ProjectBrowser() -> DockablePaneId: ...
    @property
    def PropertiesPalette() -> DockablePaneId: ...
    @property
    def RebarBrowser() -> DockablePaneId: ...
    @property
    def SystemNavigator() -> DockablePaneId: ...
    @property
    def ViewBrowser() -> DockablePaneId: ...


class DoubleClickOptions:
    @property
    def IsValidObject(self) -> bool: ...
    def GetDoubleClickOptions() -> DoubleClickOptions: ...
    def GetRenameOnSlowDoubleClickInProjectBrowser(self) -> bool: ...
    def SetRenameOnSlowDoubleClickInProjectBrowser(self, bRenameOnSlowDoubleClickInProjectBrowser: bool) -> None: ...
    def GetAction(self, target: DoubleClickTarget) -> DoubleClickAction: ...
    def SetAction(self, target: DoubleClickTarget, action: DoubleClickAction) -> None: ...
    def IsSupportedAction(self, target: DoubleClickTarget, action: DoubleClickAction) -> bool: ...
    def Dispose(self) -> None: ...


class FamilyInstancePlacingArgs:
    @property
    def ActiveView(self) -> View: ...
    @property
    def IsBanned(self) -> bool: ...
    @IsBanned.setter
    def IsBanned(self, isBanned: bool) -> None: ...
    @property
    def StatusMessage(self) -> str: ...
    @StatusMessage.setter
    def StatusMessage(self, statusMessage: str) -> None: ...
    @property
    def TooltipMessage(self) -> str: ...
    @TooltipMessage.setter
    def TooltipMessage(self, tooltipMessage: str) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class FileDialog:
    @property
    def Filter(self) -> str: ...
    @Filter.setter
    def Filter(self, filter: str) -> None: ...
    @property
    def Title(self) -> str: ...
    @Title.setter
    def Title(self, title: str) -> None: ...
    @property
    def DefaultFilterEntry(self) -> str: ...
    @DefaultFilterEntry.setter
    def DefaultFilterEntry(self, defaultFilterEntry: str) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetSelectedModelPath(self) -> ModelPath: ...
    def Show(self) -> ItemSelectionDialogResult: ...
    def IsValidFilterString(filterString: str) -> bool: ...
    def Dispose(self) -> None: ...


class FileOpenDialog(FileDialog):
    def __init__(self, filter: str): ...
    @property
    def ShowPreview(self) -> bool: ...
    @ShowPreview.setter
    def ShowPreview(self, showPreview: bool) -> None: ...


class FileSaveDialog(FileDialog):
    def __init__(self, filter: str): ...
    @property
    def InitialFileName(self) -> str: ...
    @InitialFileName.setter
    def InitialFileName(self, initialFileName: str) -> None: ...


class FilterDialog:
    @overload
    def __init__(self, doc: Document, name: str): ...
    @overload
    def __init__(self, doc: Document, filterToSelect: ElementId): ...
    @property
    def NewFilterId(self) -> ElementId: ...
    @property
    def FilterToSelect(self) -> ElementId: ...
    @FilterToSelect.setter
    def FilterToSelect(self, id: ElementId) -> None: ...
    @property
    def NewFilterName(self) -> str: ...
    @NewFilterName.setter
    def NewFilterName(self, newFilterName: str) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Show(self) -> None: ...
    def Dispose(self) -> None: ...


class IExternalResourceUIServer:
    def GetDBServerId(self) -> Guid: ...
    def HandleLoadResourceResults(self, document: Document, loadData: List[ExternalResourceLoadData]) -> None: ...
    def HandleBrowseResult(self, resultType: ExternalResourceUIBrowseResultType, browsingItemPath: str) -> None: ...


class RibbonButtonOptions:
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class RevitLinkUIUtils:
    def ReportLinkLoadResults(doc: Document, loadResults: IDictionary) -> None: ...


class SelectionUIOptions:
    @property
    def SelectFaces(self) -> bool: ...
    @SelectFaces.setter
    def SelectFaces(self, selectFaces: bool) -> None: ...
    @property
    def SelectLinks(self) -> bool: ...
    @SelectLinks.setter
    def SelectLinks(self, selectLinks: bool) -> None: ...
    @property
    def SelectUnderlay(self) -> bool: ...
    @SelectUnderlay.setter
    def SelectUnderlay(self, selectUnderlay: bool) -> None: ...
    @property
    def DragOnSelection(self) -> bool: ...
    @DragOnSelection.setter
    def DragOnSelection(self, dragOnSelection: bool) -> None: ...
    @property
    def SelectPinned(self) -> bool: ...
    @SelectPinned.setter
    def SelectPinned(self, selectPinned: bool) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetSelectionUIOptions() -> SelectionUIOptions: ...
    def ElementSelectsAsPinned(document: Document, element: Element) -> bool: ...
    def Dispose(self) -> None: ...


class SetupEnergySimulationDialog:
    def __init__(self): ...
    @property
    def ProjectName(self) -> str: ...
    @property
    def ProjectId(self) -> int: ...
    @property
    def RunName(self) -> str: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Show(self) -> SetupEnergySimulationDialogResult: ...
    def Dispose(self) -> None: ...


class TableViewUIUtils:
    def TestCellAndPromptToEditTypeParameter(tableView: TableView, sectionType: SectionType, row: int, column: int) -> bool: ...


class TextEditorOptions:
    @property
    def ShowBorder(self) -> bool: ...
    @ShowBorder.setter
    def ShowBorder(self, showBorder: bool) -> None: ...
    @property
    def ShowOpaqueBackground(self) -> bool: ...
    @ShowOpaqueBackground.setter
    def ShowOpaqueBackground(self, showOpaqueBackground: bool) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetTextEditorOptions() -> TextEditorOptions: ...
    def Dispose(self) -> None: ...


class ThinLinesOptions:
    @property
    def AreThinLinesEnabled() -> bool: ...
    @AreThinLinesEnabled.setter
    def AreThinLinesEnabled(isEnabled: bool) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class UISaveAsOptions:
    def __init__(self): ...
    @property
    def ShowOverwriteWarning(self) -> bool: ...
    @ShowOverwriteWarning.setter
    def ShowOverwriteWarning(self, showOverwriteWarning: bool) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class UIFabricationUtils:
    def SetOpenConnectorIndicatorPlanColor(color: Color) -> None: ...
    def GetOpenConnectorIndicatorPlanColor() -> Color: ...
    def SetOpenConnectorIndicatorTowardsColor(color: Color) -> None: ...
    def GetOpenConnectorIndicatorTowardsColor() -> Color: ...
    def SetOpenConnectorIndicatorAwayColor(color: Color) -> None: ...
    def GetOpenConnectorIndicatorAwayColor() -> Color: ...


class UITheme:
    Dark = 0
    Light = 1


class ItemSelectionDialogResult:
    Confirmed = 0
    Canceled = 1


class DoubleClickTarget:
    Family = 0
    SketchedElement = 1
    ViewOnSheet = 2
    Assembly = 3
    Group = 4
    ComponentStairs = 5
    OutsideViewOnSheet = 6


class DoubleClickAction:
    NoAction = 0
    EditFamily = 1
    EditType = 2
    ActivateView = 3
    EnterEditMode = 4
    DeactivateView = 5


class SetupEnergySimulationDialogResult:
    Continue = 1
    Cancel = 2



class EnumConverter(TypeConverter):
 """
 Provides a type converter to convert System.Enum objects to and from various other representations.

 

 EnumConverter(type: Type)
 """
 def CanConvertFrom(self,*__args):
  """
  CanConvertFrom(self: EnumConverter,context: ITypeDescriptorContext,sourceType: Type) -> bool

  

   Gets a value indicating whether this converter can convert an object in the given source type to 

    an enumeration object using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   sourceType: A System.Type that represents the type you wish to convert from.

   Returns: true if this converter can perform the conversion; otherwise,false.
  """
  pass
 def CanConvertTo(self,*__args):
  """
  CanConvertTo(self: EnumConverter,context: ITypeDescriptorContext,destinationType: Type) -> bool

  

   Gets a value indicating whether this converter can convert an object to the given destination 

    type using the context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   destinationType: A System.Type that represents the type you wish to convert to.

   Returns: true if this converter can perform the conversion; otherwise,false.
  """
  pass
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: EnumConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   Converts the specified value object to an enumeration object.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: An optional System.Globalization.CultureInfo. If not supplied,the current culture is assumed.

   value: The System.Object to convert.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: EnumConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   Converts the given value object to the specified destination type.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: An optional System.Globalization.CultureInfo. If not supplied,the current culture is assumed.

   value: The System.Object to convert.

   destinationType: The System.Type to convert the value to.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def GetStandardValues(self,context=None):
  """
  GetStandardValues(self: EnumConverter,context: ITypeDescriptorContext) -> StandardValuesCollection

  

   Gets a collection of standard values for the data type this validator is designed for.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   Returns: A System.ComponentModel.TypeConverter.StandardValuesCollection that holds a standard set of 

    valid values,or null if the data type does not support a standard set of values.
  """
  pass
 def GetStandardValuesExclusive(self,context=None):
  """
  GetStandardValuesExclusive(self: EnumConverter,context: ITypeDescriptorContext) -> bool

  

   Gets a value indicating whether the list of standard values returned from 

    System.ComponentModel.TypeConverter.GetStandardValues is an exclusive list using the specified 

    context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   Returns: true if the System.ComponentModel.TypeConverter.StandardValuesCollection returned from 

    System.ComponentModel.TypeConverter.GetStandardValues is an exhaustive list of possible values; 

    false if other values are possible.
  """
  pass
 def GetStandardValuesSupported(self,context=None):
  """
  GetStandardValuesSupported(self: EnumConverter,context: ITypeDescriptorContext) -> bool

  

   Gets a value indicating whether this object supports a standard set of values that can be picked 

    from a list using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   Returns: true because System.ComponentModel.TypeConverter.GetStandardValues should be called to find a 

    common set of values the object supports. This method never returns false.
  """
  pass
 def IsValid(self,*__args):
  """
  IsValid(self: EnumConverter,context: ITypeDescriptorContext,value: object) -> bool

  

   Gets a value indicating whether the given object value is valid for this type.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   value: The System.Object to test.

   Returns: true if the specified value is valid for this object; otherwise,false.
  """
  pass
 @staticmethod
 def __new__(self,type):
  """ __new__(cls: type,type: Type) """
  pass
 Comparer=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets an System.Collections.IComparer that can be used to sort the values of the enumeration.



"""

 EnumType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Specifies the type of the enumerator this converter is associated with.



"""

 Values=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a System.ComponentModel.TypeConverter.StandardValuesCollection that specifies the possible values for the enumeration.



"""





class DateTimeConverter(TypeConverter):
 """
 Provides a type converter to convert System.DateTime objects to and from various other representations.

 

 DateTimeConverter()
 """
 def CanConvertFrom(self,*__args):
  """
  CanConvertFrom(self: DateTimeConverter,context: ITypeDescriptorContext,sourceType: Type) -> bool

  

   Gets a value indicating whether this converter can convert an object in the given source type to 

    a System.DateTime using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   sourceType: A System.Type that represents the type you wish to convert from.

   Returns: true if this object can perform the conversion; otherwise,false.
  """
  pass
 def CanConvertTo(self,*__args):
  """
  CanConvertTo(self: DateTimeConverter,context: ITypeDescriptorContext,destinationType: Type) -> bool

  

   Gets a value indicating whether this converter can convert an object to the given destination 

    type using the context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   destinationType: A System.Type that represents the type you wish to convert to.

   Returns: true if this converter can perform the conversion; otherwise,false.
  """
  pass
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: DateTimeConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   Converts the given value object to a System.DateTime.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: An optional System.Globalization.CultureInfo. If not supplied,the current culture is assumed.

   value: The System.Object to convert.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: DateTimeConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   Converts the given value object to a System.DateTime using the arguments.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: An optional System.Globalization.CultureInfo. If not supplied,the current culture is assumed.

   value: The System.Object to convert.

   destinationType: The System.Type to convert the value to.

   Returns: An System.Object that represents the converted value.
  """
  pass



class Console(object):
 """ Represents the standard input,output,and error streams for console applications. This class cannot be inherited. """
 @staticmethod
 def Beep(frequency=None,duration=None):
  """
  Beep(frequency: int,duration: int)

   Plays the sound of a beep of a specified frequency and duration through the console speaker.

  

   frequency: The frequency of the beep,ranging from 37 to 32767 hertz.

   duration: The duration of the beep measured in milliseconds.

  Beep()

   Plays the sound of a beep through the console speaker.
  """
  pass
 @staticmethod
 def Clear():
  """
  Clear()

   Clears the console buffer and corresponding console window of display information.
  """
  pass
 @staticmethod
 def MoveBufferArea(sourceLeft,sourceTop,sourceWidth,sourceHeight,targetLeft,targetTop,sourceChar=None,sourceForeColor=None,sourceBackColor=None):
  """
  MoveBufferArea(sourceLeft: int,sourceTop: int,sourceWidth: int,sourceHeight: int,targetLeft: int,targetTop: int,sourceChar: Char,sourceForeColor: ConsoleColor,sourceBackColor: ConsoleColor)

   Copies a specified source area of the screen buffer to a specified destination area.

  

   sourceLeft: The leftmost column of the source area.

   sourceTop: The topmost row of the source area.

   sourceWidth: The number of columns in the source area.

   sourceHeight: The number of rows in the source area.

   targetLeft: The leftmost column of the destination area.

   targetTop: The topmost row of the destination area.

   sourceChar: The character used to fill the source area.

   sourceForeColor: The foreground color used to fill the source area.

   sourceBackColor: The background color used to fill the source area.

  MoveBufferArea(sourceLeft: int,sourceTop: int,sourceWidth: int,sourceHeight: int,targetLeft: int,targetTop: int)

   Copies a specified source area of the screen buffer to a specified destination area.

  

   sourceLeft: The leftmost column of the source area.

   sourceTop: The topmost row of the source area.

   sourceWidth: The number of columns in the source area.

   sourceHeight: The number of rows in the source area.

   targetLeft: The leftmost column of the destination area.

   targetTop: The topmost row of the destination area.
  """
  pass
 @staticmethod
 def OpenStandardError(bufferSize=None):
  """
  OpenStandardError(bufferSize: int) -> Stream

  

   Acquires the standard error stream,which is set to a specified buffer size.

  

   bufferSize: The internal stream buffer size.

   Returns: The standard error stream.

  OpenStandardError() -> Stream

  

   Acquires the standard error stream.

   Returns: The standard error stream.
  """
  pass
 @staticmethod
 def OpenStandardInput(bufferSize=None):
  """
  OpenStandardInput(bufferSize: int) -> Stream

  

   Acquires the standard input stream,which is set to a specified buffer size.

  

   bufferSize: The internal stream buffer size.

   Returns: The standard input stream.

  OpenStandardInput() -> Stream

  

   Acquires the standard input stream.

   Returns: The standard input stream.
  """
  pass
 @staticmethod
 def OpenStandardOutput(bufferSize=None):
  """
  OpenStandardOutput(bufferSize: int) -> Stream

  

   Acquires the standard output stream,which is set to a specified buffer size.

  

   bufferSize: The internal stream buffer size.

   Returns: The standard output stream.

  OpenStandardOutput() -> Stream

  

   Acquires the standard output stream.

   Returns: The standard output stream.
  """
  pass
 @staticmethod
 def Read():
  """
  Read() -> int

  

   Reads the next character from the standard input stream.

   Returns: The next character from the input stream,or negative one (-1) if there are currently no more 

    characters to be read.
  """
  pass
 @staticmethod
 def ReadKey(intercept=None):
  """
  ReadKey(intercept: bool) -> ConsoleKeyInfo

  

   Obtains the next character or function key pressed by the user. The pressed key is optionally 

    displayed in the console window.

  

  

   intercept: Determines whether to display the pressed key in the console window. true to not display the 

    pressed key; otherwise,false.

  

   Returns: A System.ConsoleKeyInfo object that describes the System.ConsoleKey constant and Unicode 

    character,if any,that correspond to the pressed console key. The System.ConsoleKeyInfo object 

    also describes,in a bitwise combination of System.ConsoleModifiers values,whether one or more 

    SHIFT,ALT,or CTRL modifier keys was pressed simultaneously with the console key.

  

  ReadKey() -> ConsoleKeyInfo

  

   Obtains the next character or function key pressed by the user. The pressed key is displayed in 

    the console window.

  

   Returns: A System.ConsoleKeyInfo object that describes the System.ConsoleKey constant and Unicode 

    character,if any,that correspond to the pressed console key. The System.ConsoleKeyInfo object 

    also describes,in a bitwise combination of System.ConsoleModifiers values,whether one or more 

    SHIFT,ALT,or CTRL modifier keys was pressed simultaneously with the console key.
  """
  pass
 @staticmethod
 def ReadLine():
  """
  ReadLine() -> str

  

   Reads the next line of characters from the standard input stream.

   Returns: The next line of characters from the input stream,or null if no more lines are available.
  """
  pass
 @staticmethod
 def ResetColor():
  """
  ResetColor()

   Sets the foreground and background console colors to their defaults.
  """
  pass
 @staticmethod
 def SetBufferSize(width,height):
  """
  SetBufferSize(width: int,height: int)

   Sets the height and width of the screen buffer area to the specified values.

  

   width: The width of the buffer area measured in columns.

   height: The height of the buffer area measured in rows.
  """
  pass
 @staticmethod
 def SetCursorPosition(left,top):
  """
  SetCursorPosition(left: int,top: int)

   Sets the position of the cursor.

  

   left: The column position of the cursor.

   top: The row position of the cursor.
  """
  pass
 @staticmethod
 def SetError(newError):
  """
  SetError(newError: TextWriter)

   Sets the System.Console.Error property to the specified System.IO.TextWriter object.

  

   newError: A stream that is the new standard error output.
  """
  pass
 @staticmethod
 def SetIn(newIn):
  """
  SetIn(newIn: TextReader)

   Sets the System.Console.In property to the specified System.IO.TextReader object.

  

   newIn: A stream that is the new standard input.
  """
  pass
 @staticmethod
 def SetOut(newOut):
  """
  SetOut(newOut: TextWriter)

   Sets the System.Console.Out property to the specified System.IO.TextWriter object.

  

   newOut: A stream that is the new standard output.
  """
  pass
 @staticmethod
 def SetWindowPosition(left,top):
  """
  SetWindowPosition(left: int,top: int)

   Sets the position of the console window relative to the screen buffer.

  

   left: The column position of the upper left  corner of the console window.

   top: The row position of the upper left corner of the console window.
  """
  pass
 @staticmethod
 def SetWindowSize(width,height):
  """
  SetWindowSize(width: int,height: int)

   Sets the height and width of the console window to the specified values.

  

   width: The width of the console window measured in columns.

   height: The height of the console window measured in rows.
  """
  pass
 @staticmethod
 def Write(*__args):
  """
  Write(value: Single)

   Writes the text representation of the specified single-precision floating-point value to the 

    standard output stream.

  

  

   value: The value to write.

  Write(value: int)

   Writes the text representation of the specified 32-bit signed integer value to the standard 

    output stream.

  

  

   value: The value to write.

  Write(value: float)

   Writes the text representation of the specified double-precision floating-point value to the 

    standard output stream.

  

  

   value: The value to write.

  Write(value: Decimal)

   Writes the text representation of the specified System.Decimal value to the standard output 

    stream.

  

  

   value: The value to write.

  Write(value: UInt32)

   Writes the text representation of the specified 32-bit unsigned integer value to the standard 

    output stream.

  

  

   value: The value to write.

  Write(value: object)

   Writes the text representation of the specified object to the standard output stream.

  

   value: The value to write,or null.

  Write(value: str)

   Writes the specified string value to the standard output stream.

  

   value: The value to write.

  Write(value: Int64)

   Writes the text representation of the specified 64-bit signed integer value to the standard 

    output stream.

  

  

   value: The value to write.

  Write(value: UInt64)

   Writes the text representation of the specified 64-bit unsigned integer value to the standard 

    output stream.

  

  

   value: The value to write.

  Write(format: str,arg0: object,arg1: object,arg2: object)

   Writes the text representation of the specified objects to the standard output stream using the 

    specified format information.

  

  

   format: A composite format string (see Remarks).

   arg0: The first object to write using format.

   arg1: The second object to write using format.

   arg2: The third object to write using format.

  Write(format: str,arg0: object,arg1: object,arg2: object,arg3: object)

   Writes the text representation of the specified objects and variable-length parameter list to 

    the standard output stream using the specified format information.

  

  

   format: A composite format string (see Remarks).

   arg0: The first object to write using format.

   arg1: The second object to write using format.

   arg2: The third object to write using format.

   arg3: The fourth object to write using format.

  Write(format: str,arg0: object)

   Writes the text representation of the specified object to the standard output stream using the 

    specified format information.

  

  

   format: A composite format string (see Remarks).

   arg0: An object to write using format.

  Write(format: str,arg0: object,arg1: object)

   Writes the text representation of the specified objects to the standard output stream using the 

    specified format information.

  

  

   format: A composite format string (see Remarks).

   arg0: The first object to write using format.

   arg1: The second object to write using format.

  Write(format: str,*arg: Array[object])

   Writes the text representation of the specified array of objects to the standard output stream 

    using the specified format information.

  

  

   format: A composite format string (see Remarks).

   arg: An array of objects to write using format.

  Write(buffer: Array[Char])

   Writes the specified array of Unicode characters to the standard output stream.

  

   buffer: A Unicode character array.

  Write(buffer: Array[Char],index: int,count: int)

   Writes the specified subarray of Unicode characters to the standard output stream.

  

   buffer: An array of Unicode characters.

   index: The starting position in buffer.

   count: The number of characters to write.

  Write(value: bool)

   Writes the text representation of the specified Boolean value to the standard output stream.

  

   value: The value to write.

  Write(value: Char)

   Writes the specified Unicode character value to the standard output stream.

  

   value: The value to write.
  """
  pass
 @staticmethod
 def WriteLine(*__args):
  """
  WriteLine(value: object)

   Writes the text representation of the specified object,followed by the current line terminator,

    to the standard output stream.

  

  

   value: The value to write.

  WriteLine(value: str)

   Writes the specified string value,followed by the current line terminator,to the standard 

    output stream.

  

  

   value: The value to write.

  WriteLine(value: Int64)

   Writes the text representation of the specified 64-bit signed integer value,followed by the 

    current line terminator,to the standard output stream.

  

  

   value: The value to write.

  WriteLine(value: UInt64)

   Writes the text representation of the specified 64-bit unsigned integer value,followed by the 

    current line terminator,to the standard output stream.

  

  

   value: The value to write.

  WriteLine(format: str,arg0: object)

   Writes the text representation of the specified object,followed by the current line terminator,

    to the standard output stream using the specified format information.

  

  

   format: A composite format string (see Remarks).

   arg0: An object to write using format.

  WriteLine(format: str,arg0: object,arg1: object,arg2: object,arg3: object)

   Writes the text representation of the specified objects and variable-length parameter list,

    followed by the current line terminator,to the standard output stream using the specified 

    format information.

  

  

   format: A composite format string (see Remarks).

   arg0: The first object to write using format.

   arg1: The second object to write using format.

   arg2: The third object to write using format.

   arg3: The fourth object to write using format.

  WriteLine(format: str,*arg: Array[object])

   Writes the text representation of the specified array of objects,followed by the current line 

    terminator,to the standard output stream using the specified format information.

  

  

   format: A composite format string (see Remarks).

   arg: An array of objects to write using format.

  WriteLine(format: str,arg0: object,arg1: object)

   Writes the text representation of the specified objects,followed by the current line 

    terminator,to the standard output stream using the specified format information.

  

  

   format: A composite format string (see Remarks).

   arg0: The first object to write using format.

   arg1: The second object to write using format.

  WriteLine(format: str,arg0: object,arg1: object,arg2: object)

   Writes the text representation of the specified objects,followed by the current line 

    terminator,to the standard output stream using the specified format information.

  

  

   format: A composite format string (see Remarks).

   arg0: The first object to write using format.

   arg1: The second object to write using format.

   arg2: The third object to write using format.

  WriteLine(value: UInt32)

   Writes the text representation of the specified 32-bit unsigned integer value,followed by the 

    current line terminator,to the standard output stream.

  

  

   value: The value to write.

  WriteLine(value: Char)

   Writes the specified Unicode character,followed by the current line terminator,value to the 

    standard output stream.

  

  

   value: The value to write.

  WriteLine(buffer: Array[Char])

   Writes the specified array of Unicode characters,followed by the current line terminator,to 

    the standard output stream.

  

  

   buffer: A Unicode character array.

  WriteLine()

   Writes the current line terminator to the standard output stream.

  WriteLine(value: bool)

   Writes the text representation of the specified Boolean value,followed by the current line 

    terminator,to the standard output stream.

  

  

   value: The value to write.

  WriteLine(buffer: Array[Char],index: int,count: int)

   Writes the specified subarray of Unicode characters,followed by the current line terminator,to 

    the standard output stream.

  

  

   buffer: An array of Unicode characters.

   index: The starting position in buffer.

   count: The number of characters to write.

  WriteLine(value: Single)

   Writes the text representation of the specified single-precision floating-point value,followed 

    by the current line terminator,to the standard output stream.

  

  

   value: The value to write.

  WriteLine(value: int)

   Writes the text representation of the specified 32-bit signed integer value,followed by the 

    current line terminator,to the standard output stream.

  

  

   value: The value to write.

  WriteLine(value: Decimal)

   Writes the text representation of the specified System.Decimal value,followed by the current 

    line terminator,to the standard output stream.

  

  

   value: The value to write.

  WriteLine(value: float)

   Writes the text representation of the specified double-precision floating-point value,followed 

    by the current line terminator,to the standard output stream.

  

  

   value: The value to write.
  """
  pass
 BackgroundColor=None
 BufferHeight=1000
 BufferWidth=127
 CancelKeyPress=None
 CapsLock=False
 CursorLeft=0
 CursorSize=25
 CursorTop=999
 CursorVisible=True
 Error=None
 ForegroundColor=None
 In=None
 InputEncoding=None
 IsErrorRedirected=False
 IsInputRedirected=False
 IsOutputRedirected=False
 KeyAvailable=False
 LargestWindowHeight=60
 LargestWindowWidth=127
 NumberLock=True
 Out=None
 OutputEncoding=None
 Title='cmd - ipy  -m ironstubs make System --folder=stubs2 --overwrite'
 TreatControlCAsInput=False
 WindowHeight=60
 WindowLeft=0
 WindowTop=940
 WindowWidth=127
 __all__=[
  'Beep',
  'CancelKeyPress',
  'Clear',
  'MoveBufferArea',
  'OpenStandardError',
  'OpenStandardInput',
  'OpenStandardOutput',
  'Read',
  'ReadKey',
  'ReadLine',
  'ResetColor',
  'SetBufferSize',
  'SetCursorPosition',
  'SetError',
  'SetIn',
  'SetOut',
  'SetWindowPosition',
  'SetWindowSize',
  'Write',
  'WriteLine',
 ]




class TraceFilter(object):
 """ Provides the base class for trace filter implementations. """
 def ShouldTrace(self,cache,source,eventType,id,formatOrMessage,args,data1,data):
  """
  ShouldTrace(self: TraceFilter,cache: TraceEventCache,source: str,eventType: TraceEventType,id: int,formatOrMessage: str,args: Array[object],data1: object,data: Array[object]) -> bool

  

   When overridden in a derived class,determines whether the trace listener should trace the event.

  

   cache: The System.Diagnostics.TraceEventCache that contains information for the trace event.

   source: The name of the source.

   eventType: One of the System.Diagnostics.TraceEventType values specifying the type of event that has caused 

    the trace.

  

   id: A trace identifier number.

   formatOrMessage: Either the format to use for writing an array of arguments specified by the args parameter,or a 

    message to write.

  

   args: An array of argument objects.

   data1: A trace data object.

   data: An array of trace data objects.

   Returns: true to trace the specified event; otherwise,false.
  """
  pass



class ImageConverter(TypeConverter):
 """
 System.Drawing.ImageConverter  is a class that can be used to convert System.Drawing.Image objects from one data type to another. Access this class through the System.ComponentModel.TypeDescriptor object.

 

 ImageConverter()
 """
 def CanConvertFrom(self,*__args):
  """
  CanConvertFrom(self: ImageConverter,context: ITypeDescriptorContext,sourceType: Type) -> bool

  

   Determines whether this System.Drawing.ImageConverter can convert an instance of a specified 

    type to an System.Drawing.Image,using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   sourceType: A System.Type that specifies the type you want to convert from.

   Returns: This method returns true if this System.Drawing.ImageConverter can perform the conversion; 

    otherwise,false.
  """
  pass
 def CanConvertTo(self,*__args):
  """
  CanConvertTo(self: ImageConverter,context: ITypeDescriptorContext,destinationType: Type) -> bool

  

   Determines whether this System.Drawing.ImageConverter can convert an System.Drawing.Image to an 

    instance of a specified type,using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   destinationType: A System.Type that specifies the type you want to convert to.

   Returns: This method returns true if this System.Drawing.ImageConverter can perform the conversion; 

    otherwise,false.
  """
  pass
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: ImageConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   Converts a specified object to an System.Drawing.Image.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: A System.Globalization.CultureInfo that holds information about a specific culture.

   value: The System.Object to be converted.

   Returns: If this method succeeds,it returns the System.Drawing.Image that it created by converting the 

    specified object. Otherwise,it throws an exception.
  """
  pass
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: ImageConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   Converts an System.Drawing.Image (or an object that can be cast to an System.Drawing.Image) to 

    the specified type.

  

  

   context: A formatter context. This object can be used to get more information about the environment this 

    converter is being called from. This may be null,so you should always check. Also,properties 

    on the context object may also return null.

  

   culture: A System.Globalization.CultureInfo object that specifies formatting conventions used by a 

    particular culture.

  

   value: The System.Drawing.Image to convert.

   destinationType: The System.Type to convert the System.Drawing.Image to.

   Returns: This method returns the converted object.
  """
  pass
 def GetProperties(self,*__args):
  """
  GetProperties(self: ImageConverter,context: ITypeDescriptorContext,value: object,attributes: Array[Attribute]) -> PropertyDescriptorCollection

  

   Gets the set of properties for this type.

  

   context: A type descriptor through which additional context can be provided.

   value: The value of the object to get the properties for.

   attributes: An array of System.Attribute objects that describe the properties.

   Returns: The set of properties that should be exposed for this data type. If no properties should be 

    exposed,this can return null. The default implementation always returns null.
  """
  pass
 def GetPropertiesSupported(self,context=None):
  """
  GetPropertiesSupported(self: ImageConverter,context: ITypeDescriptorContext) -> bool

  

   Indicates whether this object supports properties. By default,this is false.

  

   context: A type descriptor through which additional context can be provided.

   Returns: This method returns true if the erload:System.Drawing.ImageConverter.GetProperties method should 

    be called to find the properties of this object.
  """
  pass



class DataGridViewRowsAddedEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.DataGridView.RowsAdded event.

 

 DataGridViewRowsAddedEventArgs(rowIndex: int,rowCount: int)
 """
 @staticmethod
 def __new__(self,rowIndex,rowCount):
  """ __new__(cls: type,rowIndex: int,rowCount: int) """
  pass
 RowCount=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of rows that have been added.



Get: RowCount(self: DataGridViewRowsAddedEventArgs) -> int



"""

 RowIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index of the first added row.



Get: RowIndex(self: DataGridViewRowsAddedEventArgs) -> int



"""





"""
Type Casting Utilities

"""

import rpw
from rpw import revit, DB
from rpw.base import BaseObjectWrapper
from rpw.db.builtins import BicEnum
from rpw.utils.dotnet import List
from rpw.exceptions import RpwTypeError


def to_element_id(element_reference):
    """
    Coerces Element References (Element, ElementId, ...) to Element Id

    >>> from rpw.utils.coerce import to_element_id
    >>> to_element_id(SomeElement)
    <Element Id>

    """
    if hasattr(element_reference, 'Id'):
        element_id = element_reference.Id
    elif isinstance(element_reference, DB.Reference):
        element_id = element_reference.ElementId
    elif isinstance(element_reference, int):
        element_id = DB.ElementId(element_reference)
    elif isinstance(element_reference, DB.ElementId):
        element_id = element_reference
    elif element_reference == DB.ElementId.InvalidElementId:
        element_id = element_reference
    else:
        raise RpwTypeError('Element, ElementId, or int', type(element_reference))
    return element_id


def to_element_ids(element_references):
    """
    Coerces an element or list of elements into element ids.
    Elements remain unchanged.
    This will always return a list, even if only one element is passed.

    >>> from rpw.utils.coerce import to_element_ids
    >>> to_element_ids(DB.Element)
    [ DB.ElementId ]
    >>> to_element_ids(20001)
    [ DB.ElementId ]
    >>> to_element_ids([20001, 20003])
    [ DB.ElementId, DB.ElementId ]

    Args:
        elements (``DB.Element``): Iterable list (``list`` or ``set``)
                                   or single of ``Element``, ``int``.

    Returns:
        [``DB.ElementId``, ... ]: List of Element Ids.
    """
    element_references = to_iterable(element_references)
    return [to_element_id(e_ref) for e_ref in element_references]

# TODO: Add case to unwrap rpw elements
def to_element(element_reference, doc=revit.doc):
    """ Same as to_elements but for a single object """
    if isinstance(element_reference, DB.Element):
        element = element_reference
    elif isinstance(element_reference, DB.ElementId):
        element = doc.GetElement(element_reference)
    elif isinstance(element_reference, DB.Reference):
        element = doc.GetElement(element_reference)
    elif isinstance(element_reference, int):
        element = doc.GetElement(DB.ElementId(element_reference))
    elif hasattr(element_reference, 'unwrap'):
        element = element_reference.unwrap()
    else:
        raise RpwTypeError('Element, ElementId, or int', type(element_reference))
    return element


def to_elements(element_references, doc=revit.doc):
    """
    Coerces element reference (``int``, or ``ElementId``) into ``DB.Element``.
    Remains unchanged if it's already ``DB.Element``.
    Accepts single object or lists.

    >>> from rpw.utils.coerce import to_elements
    >>> to_elements(DB.ElementId)
    [ DB.Element ]
    >>> to_elements(20001)
    [ DB.Element ]
    >>> to_elements([20001, 20003])
    [ DB.Element, DB.Element ]

    Args:
        element_references ([``DB.ElementId``, ``int``, ``DB.Element``]): Element Reference,
                                                                          single or list

    Returns:
        [``DB.Element``]: Elements
    """
    element_references = to_iterable(element_references)
    return [to_element(e_ref) for e_ref in element_references]


def to_class(class_reference):
    """ Coerces a class or class reference to a Class.

    >>> from rpw.utils.coerce import to_class
    >>> to_class('Wall')
    [ DB.Wall ]
    >>> to_class(Wall)
    [ DB.Wall ]

    Args:
        class_reference ([``DB.Wall``, ``str``]): Class Reference or class name

    Returns:
        [``type``]: Class
    """
    if isinstance(class_reference, str):
        return getattr(DB, class_reference)
    if isinstance(class_reference, type):
        return class_reference
    raise RpwTypeError('Class Type, Class Type Name', type(class_reference))


def to_category(category_reference, fuzzy=True):
    """ Coerces a category, category name or category id to a BuiltInCategory.

    >>> from rpw.utils.coerce import to_category
    >>> to_category('OST_Walls')
    BuiltInCategory.OST_Walls
    >>> to_category('Walls')
    BuiltInCategory.OST_Walls
    >>> to_category(BuiltInCategory.OST_Walls)
    BuiltInCategory.OST_Walls

    Args:
        cateagory_reference ([``DB.BuiltInCategory``, ``str``, ``CategoryId``]): Category Reference
                                                                                 or Name

    Returns:
        [``BuiltInCategory``]: BuiltInCategory
    """
    if isinstance(category_reference, DB.BuiltInCategory):
        return category_reference
    if isinstance(category_reference, str):
        if fuzzy:
            return BicEnum.fuzzy_get(category_reference)
        else:
            return BicEnum.get(category_reference)
    if isinstance(category_reference, DB.ElementId):
        return BicEnum.from_category_id(category_reference)
    raise RpwTypeError('Category Type, Category Type Name',
                       type(category_reference))


def to_category_id(category_reference, fuzzy=True):
    """
    Coerces a category, category name or category id to a Category Id.

    >>> from rpw.utils.coerce import to_category_id
    >>> to_category_id('OST_Walls')
    <ElementId>
    >>> to_category_id('Wall')
    <ElementId>
    >>> to_category_id(BuiltInCategory.OST_Walls)
    <ElementId>

    Args:
        cateagory_reference ([``DB.BuiltInCategory``, ``str``, ``CategoryId``]): Category Reference
                                                                                 or Name

    Returns:
        [``DB.ElementId``]: ElementId of Category
    """
    category_enum = to_category(category_reference)
    return DB.ElementId(category_enum)


def to_iterable(item_or_iterable):
    """
    Ensures input is iterable

    >>> from rpw.utils.coerce import to_iterable
    >>> to_iterable(SomeElement)
    [SomeElement]

    Args:
        any (iterable, non-iterable)

    Returns:
        (`iterable`): Same as input
    """
    if hasattr(item_or_iterable, '__iter__'):
        return item_or_iterable
    else:
        return [item_or_iterable]


def to_pascal_case(snake_str):
    """ Converts Snake Case to Pascal Case

    >>> to_pascal_case('family_name')
    'FamilyName'
    """
    components = snake_str.split('_')
    return "".join(x.title() for x in components)


# def dictioary_to_string(dictionary):
#     """ Makes a string with key:value pairs from a dictionary

#     >>> dictionary_to_string({'name': 'value'})
#     'name:value'
#     >>> dictionary_to_string({'name': 'value', 'id':5})
#     'name:value id:5'
#     """
#     return ' '.join(['{0}:{1}'.format(k, v) for k, v in dictionary.iteritems()])



class CustomTypeDescriptor(object,ICustomTypeDescriptor):
 """ Provides a simple default implementation of the System.ComponentModel.ICustomTypeDescriptor interface. """
 def GetAttributes(self):
  """
  GetAttributes(self: CustomTypeDescriptor) -> AttributeCollection

  

   Returns a collection of custom attributes for the type represented by this type descriptor.

   Returns: An System.ComponentModel.AttributeCollection containing the attributes for the type. The default 

    is System.ComponentModel.AttributeCollection.Empty.
  """
  pass
 def GetClassName(self):
  """
  GetClassName(self: CustomTypeDescriptor) -> str

  

   Returns the fully qualified name of the class represented by this type descriptor.

   Returns: A System.String containing the fully qualified class name of the type this type descriptor is 

    describing. The default is null.
  """
  pass
 def GetComponentName(self):
  """
  GetComponentName(self: CustomTypeDescriptor) -> str

  

   Returns the name of the class represented by this type descriptor.

   Returns: A System.String containing the name of the component instance this type descriptor is 

    describing. The default is null.
  """
  pass
 def GetConverter(self):
  """
  GetConverter(self: CustomTypeDescriptor) -> TypeConverter

  

   Returns a type converter for the type represented by this type descriptor.

   Returns: A System.ComponentModel.TypeConverter for the type represented by this type descriptor. The 

    default is a newly created System.ComponentModel.TypeConverter.
  """
  pass
 def GetDefaultEvent(self):
  """
  GetDefaultEvent(self: CustomTypeDescriptor) -> EventDescriptor

  

   Returns the event descriptor for the default event of the object represented by this type 

    descriptor.

  

   Returns: The System.ComponentModel.EventDescriptor for the default event on the object represented by 

    this type descriptor. The default is null.
  """
  pass
 def GetDefaultProperty(self):
  """
  GetDefaultProperty(self: CustomTypeDescriptor) -> PropertyDescriptor

  

   Returns the property descriptor for the default property of the object represented by this type 

    descriptor.

  

   Returns: A System.ComponentModel.PropertyDescriptor for the default property on the object represented by 

    this type descriptor. The default is null.
  """
  pass
 def GetEditor(self,editorBaseType):
  """
  GetEditor(self: CustomTypeDescriptor,editorBaseType: Type) -> object

  

   Returns an editor of the specified type that is to be associated with the class represented by 

    this type descriptor.

  

  

   editorBaseType: The base type of the editor to retrieve.

   Returns: An editor of the given type that is to be associated with the class represented by this type 

    descriptor. The default is null.
  """
  pass
 def GetEvents(self,attributes=None):
  """
  GetEvents(self: CustomTypeDescriptor,attributes: Array[Attribute]) -> EventDescriptorCollection

  

   Returns a filtered collection of event descriptors for the object represented by this type 

    descriptor.

  

  

   attributes: An array of attributes to use as a filter. This can be null.

   Returns: An System.ComponentModel.EventDescriptorCollection containing the event descriptions for the 

    object represented by this type descriptor. The default is 

    System.ComponentModel.EventDescriptorCollection.Empty.

  

  GetEvents(self: CustomTypeDescriptor) -> EventDescriptorCollection

  

   Returns a collection of event descriptors for the object represented by this type descriptor.

   Returns: An System.ComponentModel.EventDescriptorCollection containing the event descriptors for the 

    object represented by this type descriptor. The default is 

    System.ComponentModel.EventDescriptorCollection.Empty.
  """
  pass
 def GetProperties(self,attributes=None):
  """
  GetProperties(self: CustomTypeDescriptor) -> PropertyDescriptorCollection

  

   Returns a collection of property descriptors for the object represented by this type descriptor.

   Returns: A System.ComponentModel.PropertyDescriptorCollection containing the property descriptions for 

    the object represented by this type descriptor. The default is 

    System.ComponentModel.PropertyDescriptorCollection.Empty.

  

  GetProperties(self: CustomTypeDescriptor,attributes: Array[Attribute]) -> PropertyDescriptorCollection

  

   Returns a filtered collection of property descriptors for the object represented by this type 

    descriptor.

  

  

   attributes: An array of attributes to use as a filter. This can be null.

   Returns: A System.ComponentModel.PropertyDescriptorCollection containing the property descriptions for 

    the object represented by this type descriptor. The default is 

    System.ComponentModel.PropertyDescriptorCollection.Empty.
  """
  pass
 def GetPropertyOwner(self,pd):
  """
  GetPropertyOwner(self: CustomTypeDescriptor,pd: PropertyDescriptor) -> object

  

   Returns an object that contains the property described by the specified property descriptor.

  

   pd: The property descriptor for which to retrieve the owning object.

   Returns: An System.Object that owns the given property specified by the type descriptor. The default is 

    null.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """
  __new__(cls: type)

  __new__(cls: type,parent: ICustomTypeDescriptor)
  """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass



class ModuleHandle(object):
 """ Represents a runtime handle for a module. """
 def Equals(self,*__args):
  """
  Equals(self: ModuleHandle,handle: ModuleHandle) -> bool

  

   Returns a System.Boolean value indicating whether the specified System.ModuleHandle structure is 

    equal to the current System.ModuleHandle.

  

  

   handle: The System.ModuleHandle structure to be compared with the current System.ModuleHandle.

   Returns: true if handle is equal to the current System.ModuleHandle structure; otherwise false.

  Equals(self: ModuleHandle,obj: object) -> bool

  

   Returns a System.Boolean value indicating whether the specified object is a System.ModuleHandle 

    structure,and equal to the current System.ModuleHandle.

  

  

   obj: The object to be compared with the current System.ModuleHandle structure.

   Returns: true if obj is a System.ModuleHandle structure,and is equal to the current System.ModuleHandle 

    structure; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """ GetHashCode(self: ModuleHandle) -> int """
  pass
 def GetRuntimeFieldHandleFromMetadataToken(self,fieldToken):
  """
  GetRuntimeFieldHandleFromMetadataToken(self: ModuleHandle,fieldToken: int) -> RuntimeFieldHandle

  

   Returns a runtime handle for the field identified by the specified metadata token.

  

   fieldToken: A metadata token that identifies a field in the module.

   Returns: A System.RuntimeFieldHandle for the field identified by fieldToken.
  """
  pass
 def GetRuntimeMethodHandleFromMetadataToken(self,methodToken):
  """
  GetRuntimeMethodHandleFromMetadataToken(self: ModuleHandle,methodToken: int) -> RuntimeMethodHandle

  

   Returns a runtime method handle for the method or constructor identified by the specified 

    metadata token.

  

  

   methodToken: A metadata token that identifies a method or constructor in the module.

   Returns: A System.RuntimeMethodHandle for the method or constructor identified by methodToken.
  """
  pass
 def GetRuntimeTypeHandleFromMetadataToken(self,typeToken):
  """
  GetRuntimeTypeHandleFromMetadataToken(self: ModuleHandle,typeToken: int) -> RuntimeTypeHandle

  

   Returns a runtime type handle for the type identified by the specified metadata token.

  

   typeToken: A metadata token that identifies a type in the module.

   Returns: A System.RuntimeTypeHandle for the type identified by typeToken.
  """
  pass
 def ResolveFieldHandle(self,fieldToken,typeInstantiationContext=None,methodInstantiationContext=None):
  """
  ResolveFieldHandle(self: ModuleHandle,fieldToken: int,typeInstantiationContext: Array[RuntimeTypeHandle],methodInstantiationContext: Array[RuntimeTypeHandle]) -> RuntimeFieldHandle

  

   Returns a runtime field handle for the field identified by the specified metadata token,

    specifying the generic type arguments of the type and method where the token is in scope.

  

  

   fieldToken: A metadata token that identifies a field in the module.

   typeInstantiationContext: An array of System.RuntimeTypeHandle structures representing the generic type arguments of the 

    type where the token is in scope,or null if that type is not generic.

  

   methodInstantiationContext: An array of System.RuntimeTypeHandle structures representing the generic type arguments of the 

    method where the token is in scope,or null if that method is not generic.

  

   Returns: A System.RuntimeFieldHandle for the field identified by fieldToken.

  ResolveFieldHandle(self: ModuleHandle,fieldToken: int) -> RuntimeFieldHandle

  

   Returns a runtime handle for the field identified by the specified metadata token.

  

   fieldToken: A metadata token that identifies a field in the module.

   Returns: A System.RuntimeFieldHandle for the field identified by fieldToken.
  """
  pass
 def ResolveMethodHandle(self,methodToken,typeInstantiationContext=None,methodInstantiationContext=None):
  """
  ResolveMethodHandle(self: ModuleHandle,methodToken: int,typeInstantiationContext: Array[RuntimeTypeHandle],methodInstantiationContext: Array[RuntimeTypeHandle]) -> RuntimeMethodHandle

  

   Returns a runtime method handle for the method or constructor identified by the specified 

    metadata token,specifying the generic type arguments of the type and method where the token is 

    in scope.

  

  

   methodToken: A metadata token that identifies a method or constructor in the module.

   typeInstantiationContext: An array of System.RuntimeTypeHandle structures representing the generic type arguments of the 

    type where the token is in scope,or null if that type is not generic.

  

   methodInstantiationContext: An array of System.RuntimeTypeHandle structures representing the generic type arguments of the 

    method where the token is in scope,or null if that method is not generic.

  

   Returns: A System.RuntimeMethodHandle for the method or constructor identified by methodToken.

  ResolveMethodHandle(self: ModuleHandle,methodToken: int) -> RuntimeMethodHandle

  

   Returns a runtime method handle for the method or constructor identified by the specified 

    metadata token.

  

  

   methodToken: A metadata token that identifies a method or constructor in the module.

   Returns: A System.RuntimeMethodHandle for the method or constructor identified by methodToken.
  """
  pass
 def ResolveTypeHandle(self,typeToken,typeInstantiationContext=None,methodInstantiationContext=None):
  """
  ResolveTypeHandle(self: ModuleHandle,typeToken: int,typeInstantiationContext: Array[RuntimeTypeHandle],methodInstantiationContext: Array[RuntimeTypeHandle]) -> RuntimeTypeHandle

  

   Returns a runtime type handle for the type identified by the specified metadata token,

    specifying the generic type arguments of the type and method where the token is in scope.

  

  

   typeToken: A metadata token that identifies a type in the module.

   typeInstantiationContext: An array of System.RuntimeTypeHandle structures representing the generic type arguments of the 

    type where the token is in scope,or null if that type is not generic.

  

   methodInstantiationContext: An array of System.RuntimeTypeHandle structures objects representing the generic type arguments 

    of the method where the token is in scope,or null if that method is not generic.

  

   Returns: A System.RuntimeTypeHandle for the type identified by typeToken.

  ResolveTypeHandle(self: ModuleHandle,typeToken: int) -> RuntimeTypeHandle

  

   Returns a runtime type handle for the type identified by the specified metadata token.

  

   typeToken: A metadata token that identifies a type in the module.

   Returns: A System.RuntimeTypeHandle for the type identified by typeToken.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __ne__(self,*args):
  pass
 MDStreamVersion=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the metadata stream version.



Get: MDStreamVersion(self: ModuleHandle) -> int



"""


 EmptyHandle=None




class EventLog(Component,IComponent,IDisposable,ISupportInitialize):
 """
 Provides interaction with Windows event logs.

 

 EventLog()

 EventLog(logName: str)

 EventLog(logName: str,machineName: str)

 EventLog(logName: str,machineName: str,source: str)
 """
 def BeginInit(self):
  """
  BeginInit(self: EventLog)

   Begins the initialization of an System.Diagnostics.EventLog used on a form or used by another 

    component. The initialization occurs at runtime.
  """
  pass
 def Clear(self):
  """
  Clear(self: EventLog)

   Removes all entries from the event log.
  """
  pass
 def Close(self):
  """
  Close(self: EventLog)

   Closes the event log and releases read and write handles.
  """
  pass
 @staticmethod
 def CreateEventSource(*__args):
  """
  CreateEventSource(sourceData: EventSourceCreationData)

   Establishes a valid event source for writing localized event messages,using the specified 

    configuration properties for the event source and the corresponding event log.

  

  

   sourceData: The configuration properties for the event source and its target event log.

  CreateEventSource(source: str,logName: str,machineName: str)

   Establishes the specified source name as a valid event source for writing entries to a log on 

    the specified computer. This method can also be used to create a new custom log on the specified 

    computer.

  

  

   source: The source by which the application is registered on the specified computer.

   logName: The name of the log the source's entries are written to. Possible values include Application,

    System,or a custom event log. If you do not specify a value,logName defaults to Application.

  

   machineName: The name of the computer to register this event source with,or "." for the local computer.

  CreateEventSource(source: str,logName: str)

   Establishes the specified source name as a valid event source for writing entries to a log on 

    the local computer. This method can also create a new custom log on the local computer.

  

  

   source: The source name by which the application is registered on the local computer.

   logName: The name of the log the source's entries are written to. Possible values include Application,

    System,or a custom event log.
  """
  pass
 @staticmethod
 def Delete(logName,machineName=None):
  """
  Delete(logName: str,machineName: str)

   Removes an event log from the specified computer.

  

   logName: The name of the log to delete. Possible values include: Application,Security,System,and any 

    custom event logs on the specified computer.

  

   machineName: The name of the computer to delete the log from,or "." for the local computer.

  Delete(logName: str)

   Removes an event log from the local computer.

  

   logName: The name of the log to delete. Possible values include: Application,Security,System,and any 

    custom event logs on the computer.
  """
  pass
 @staticmethod
 def DeleteEventSource(source,machineName=None):
  """
  DeleteEventSource(source: str,machineName: str)

   Removes the application's event source registration from the specified computer.

  

   source: The name by which the application is registered in the event log system.

   machineName: The name of the computer to remove the registration from,or "." for the local computer.

  DeleteEventSource(source: str)

   Removes the event source registration from the event log of the local computer.

  

   source: The name by which the application is registered in the event log system.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: EventLog,disposing: bool)

   Releases the unmanaged resources used by the System.Diagnostics.EventLog,and optionally 

    releases the managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def EndInit(self):
  """
  EndInit(self: EventLog)

   Ends the initialization of an System.Diagnostics.EventLog used on a form or by another 

    component. The initialization occurs at runtime.
  """
  pass
 @staticmethod
 def Exists(logName,machineName=None):
  """
  Exists(logName: str,machineName: str) -> bool

  

   Determines whether the log exists on the specified computer.

  

   logName: The log for which to search. Possible values include: Application,Security,System,other 

    application-specific logs (such as those associated with Active Directory),or any custom log on 

    the computer.

  

   machineName: The name of the computer on which to search for the log,or "." for the local computer.

   Returns: true if the log exists on the specified computer; otherwise,false.

  Exists(logName: str) -> bool

  

   Determines whether the log exists on the local computer.

  

   logName: The name of the log to search for. Possible values include: Application,Security,System,other 

    application-specific logs (such as those associated with Active Directory),or any custom log on 

    the computer.

  

   Returns: true if the log exists on the local computer; otherwise,false.
  """
  pass
 @staticmethod
 def GetEventLogs(machineName=None):
  """
  GetEventLogs(machineName: str) -> Array[EventLog]

  

   Searches for all event logs on the given computer and creates an array of 

    System.Diagnostics.EventLog objects that contain the list.

  

  

   machineName: The computer on which to search for event logs.

   Returns: An array of type System.Diagnostics.EventLog that represents the logs on the given computer.

  GetEventLogs() -> Array[EventLog]

  

   Searches for all event logs on the local computer and creates an array of 

    System.Diagnostics.EventLog objects that contain the list.

  

   Returns: An array of type System.Diagnostics.EventLog that represents the logs on the local computer.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Component,service: Type) -> object

  

   Returns an object that represents a service provided by the System.ComponentModel.Component or 

    by its System.ComponentModel.Container.

  

  

   service: A service provided by the System.ComponentModel.Component.

   Returns: An System.Object that represents a service provided by the System.ComponentModel.Component,or 

    null if the System.ComponentModel.Component does not provide the specified service.
  """
  pass
 @staticmethod
 def LogNameFromSourceName(source,machineName):
  """
  LogNameFromSourceName(source: str,machineName: str) -> str

  

   Gets the name of the log to which the specified source is registered.

  

   source: The name of the event source.

   machineName: The name of the computer on which to look,or "." for the local computer.

   Returns: The name of the log associated with the specified source in the registry.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def ModifyOverflowPolicy(self,action,retentionDays):
  """
  ModifyOverflowPolicy(self: EventLog,action: OverflowAction,retentionDays: int)

   Changes the configured behavior for writing new entries when the event log reaches its maximum 

    file size.

  

  

   action: The overflow behavior for writing new entries to the event log.

   retentionDays: The minimum number of days each event log entry is retained. This parameter is used only if 

    action is set to System.Diagnostics.OverflowAction.OverwriteOlder.
  """
  pass
 def RegisterDisplayName(self,resourceFile,resourceId):
  """
  RegisterDisplayName(self: EventLog,resourceFile: str,resourceId: Int64)

   Specifies the localized name of the event log,which is displayed in the server Event Viewer.

  

   resourceFile: The fully specified path to a localized resource file.

   resourceId: The resource identifier that indexes a localized string within the resource file.
  """
  pass
 @staticmethod
 def SourceExists(source,machineName=None):
  """
  SourceExists(source: str,machineName: str) -> bool

  

   Determines whether an event source is registered on a specified computer.

  

   source: The name of the event source.

   machineName: The name the computer on which to look,or "." for the local computer.

   Returns: true if the event source is registered on the given computer; otherwise,false.

  SourceExists(source: str) -> bool

  

   Determines whether an event source is registered on the local computer.

  

   source: The name of the event source.

   Returns: true if the event source is registered on the local computer; otherwise,false.
  """
  pass
 def WriteEntry(self,*__args):
  """
  WriteEntry(self: EventLog,message: str,type: EventLogEntryType,eventID: int,category: Int16)

   Writes an entry with the given message text,application-defined event identifier,and 

    application-defined category to the event log.

  

  

   message: The string to write to the event log.

   type: One of the System.Diagnostics.EventLogEntryType values.

   eventID: The application-specific identifier for the event.

   category: The application-specific subcategory associated with the message.

  WriteEntry(source: str,message: str,type: EventLogEntryType,eventID: int)

   Writes an entry with the given message text and application-defined event identifier to the 

    event log,using the specified registered event source.

  

  

   source: The source by which the application is registered on the specified computer.

   message: The string to write to the event log.

   type: One of the System.Diagnostics.EventLogEntryType values.

   eventID: The application-specific identifier for the event.

  WriteEntry(source: str,message: str,type: EventLogEntryType,eventID: int,category: Int16)

   Writes an entry with the given message text,application-defined event identifier,and 

    application-defined category to the event log,using the specified registered event source. The 

    category can be used by the Event Viewer to filter events in the log.

  

  

   source: The source by which the application is registered on the specified computer.

   message: The string to write to the event log.

   type: One of the System.Diagnostics.EventLogEntryType values.

   eventID: The application-specific identifier for the event.

   category: The application-specific subcategory associated with the message.

  WriteEntry(self: EventLog,message: str,type: EventLogEntryType,eventID: int,category: Int16,rawData: Array[Byte])

   Writes an entry with the given message text,application-defined event identifier,and 

    application-defined category to the event log,and appends binary data to the message.

  

  

   message: The string to write to the event log.

   type: One of the System.Diagnostics.EventLogEntryType values.

   eventID: The application-specific identifier for the event.

   category: The application-specific subcategory associated with the message.

   rawData: An array of bytes that holds the binary data associated with the entry.

  WriteEntry(source: str,message: str,type: EventLogEntryType,eventID: int,category: Int16,rawData: Array[Byte])

   Writes an entry with the given message text,application-defined event identifier,and 

    application-defined category to the event log (using the specified registered event source) and 

    appends binary data to the message.

  

  

   source: The source by which the application is registered on the specified computer.

   message: The string to write to the event log.

   type: One of the System.Diagnostics.EventLogEntryType values.

   eventID: The application-specific identifier for the event.

   category: The application-specific subcategory associated with the message.

   rawData: An array of bytes that holds the binary data associated with the entry.

  WriteEntry(source: str,message: str)

   Writes an information type entry with the given message text to the event log,using the 

    specified registered event source.

  

  

   source: The source by which the application is registered on the specified computer.

   message: The string to write to the event log.

  WriteEntry(self: EventLog,message: str)

   Writes an information type entry,with the given message text,to the event log.

  

   message: The string to write to the event log.

  WriteEntry(self: EventLog,message: str,type: EventLogEntryType)

   Writes an error,warning,information,success audit,or failure audit entry with the given 

    message text to the event log.

  

  

   message: The string to write to the event log.

   type: One of the System.Diagnostics.EventLogEntryType values.

  WriteEntry(self: EventLog,message: str,type: EventLogEntryType,eventID: int)

   Writes an entry with the given message text and application-defined event identifier to the 

    event log.

  

  

   message: The string to write to the event log.

   type: One of the System.Diagnostics.EventLogEntryType values.

   eventID: The application-specific identifier for the event.

  WriteEntry(source: str,message: str,type: EventLogEntryType)

   Writes an error,warning,information,success audit,or failure audit entry with the given 

    message text to the event log,using the specified registered event source.

  

  

   source: The source by which the application is registered on the specified computer.

   message: The string to write to the event log.

   type: One of the System.Diagnostics.EventLogEntryType values.
  """
  pass
 def WriteEvent(self,*__args):
  """
  WriteEvent(source: str,instance: EventInstance,*values: Array[object])

   Writes an event log entry with the given event data and message replacement strings,using the 

    specified registered event source.

  

  

   source: The name of the event source registered for the application on the specified computer.

   instance: An System.Diagnostics.EventInstance instance that represents a localized event log entry.

   values: An array of strings to merge into the message text of the event log entry.

  WriteEvent(source: str,instance: EventInstance,data: Array[Byte],*values: Array[object])

   Writes an event log entry with the given event data,message replacement strings,and associated 

    binary data,and using the specified registered event source.

  

  

   source: The name of the event source registered for the application on the specified computer.

   instance: An System.Diagnostics.EventInstance instance that represents a localized event log entry.

   data: An array of bytes that holds the binary data associated with the entry.

   values: An array of strings to merge into the message text of the event log entry.

  WriteEvent(self: EventLog,instance: EventInstance,*values: Array[object])

   Writes a localized entry to the event log.

  

   instance: An System.Diagnostics.EventInstance instance that represents a localized event log entry.

   values: An array of strings to merge into the message text of the event log entry.

  WriteEvent(self: EventLog,instance: EventInstance,data: Array[Byte],*values: Array[object])

   Writes an event log entry with the given event data,message replacement strings,and associated 

    binary data.

  

  

   instance: An System.Diagnostics.EventInstance instance that represents a localized event log entry.

   data: An array of bytes that holds the binary data associated with the entry.

   values: An array of strings to merge into the message text of the event log entry.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,logName=None,machineName=None,source=None):
  """
  __new__(cls: type)

  __new__(cls: type,logName: str)

  __new__(cls: type,logName: str,machineName: str)

  __new__(cls: type,logName: str,machineName: str,source: str)
  """
  pass
 def __str__(self,*args):
  pass
 CanRaiseEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the component can raise an event.



"""

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.ComponentModel.Component is currently in design mode.



"""

 EnableRaisingEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the System.Diagnostics.EventLog receives System.Diagnostics.EventLog.EntryWritten event notifications.



Get: EnableRaisingEvents(self: EventLog) -> bool



Set: EnableRaisingEvents(self: EventLog)=value

"""

 Entries=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the contents of the event log.



Get: Entries(self: EventLog) -> EventLogEntryCollection



"""

 Events=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of event handlers that are attached to this System.ComponentModel.Component.



"""

 Log=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the log to read from or write to.



Get: Log(self: EventLog) -> str



Set: Log(self: EventLog)=value

"""

 LogDisplayName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the event log's friendly name.



Get: LogDisplayName(self: EventLog) -> str



"""

 MachineName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the computer on which to read or write events.



Get: MachineName(self: EventLog) -> str



Set: MachineName(self: EventLog)=value

"""

 MaximumKilobytes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the maximum event log size in kilobytes.



Get: MaximumKilobytes(self: EventLog) -> Int64



Set: MaximumKilobytes(self: EventLog)=value

"""

 MinimumRetentionDays=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of days to retain entries in the event log.



Get: MinimumRetentionDays(self: EventLog) -> int



"""

 OverflowAction=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the configured behavior for storing new entries when the event log reaches its maximum log file size.



Get: OverflowAction(self: EventLog) -> OverflowAction



"""

 Source=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the source name to register and use when writing to the event log.



Get: Source(self: EventLog) -> str



Set: Source(self: EventLog)=value

"""

 SynchronizingObject=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the object used to marshal the event handler calls issued as a result of an System.Diagnostics.EventLog entry written event.



Get: SynchronizingObject(self: EventLog) -> ISynchronizeInvoke



Set: SynchronizingObject(self: EventLog)=value

"""


 EntryWritten=None




class DataGridViewRowErrorTextNeededEventArgs(EventArgs):
 """ Provides data for the System.Windows.Forms.DataGridView.RowErrorTextNeeded event of a System.Windows.Forms.DataGridView control. """
 ErrorText=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the error text for the row.



Get: ErrorText(self: DataGridViewRowErrorTextNeededEventArgs) -> str



Set: ErrorText(self: DataGridViewRowErrorTextNeededEventArgs)=value

"""

 RowIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the row that raised the System.Windows.Forms.DataGridView.RowErrorTextNeeded event.



Get: RowIndex(self: DataGridViewRowErrorTextNeededEventArgs) -> int



"""





class TextSearch(DependencyObject):
 """ Enables a user to quickly access items in a set by typing prefixes of strings. """
 @staticmethod
 def GetText(element):
  """
  GetText(element: DependencyObject) -> str

  

   Returns the string to that identifies the specified item.

  

   element: The element from which the property value is read.

   Returns: The string that identifies the specified item.
  """
  pass
 @staticmethod
 def GetTextPath(element):
  """
  GetTextPath(element: DependencyObject) -> str

  

   Returns the name of the property that identifies an item in the specified element's collection.

  

   element: The element from which the property value is read.

   Returns: The name of the property that identifies the item to the user.
  """
  pass
 @staticmethod
 def SetText(element,text):
  """
  SetText(element: DependencyObject,text: str)

   Writes the System.Windows.Controls.TextSearch.Text�attached property value to the specified 

    element.

  

  

   element: The element to which the property value is written.

   text: The string that identifies the item.
  """
  pass
 @staticmethod
 def SetTextPath(element,path):
  """
  SetTextPath(element: DependencyObject,path: str)

   Writes the System.Windows.Controls.TextSearch.TextPath�attached property to the specified 

    element.

  

  

   element: The element to which the property value is written.

   path: The name of the property that identifies an item.
  """
  pass
 TextPathProperty=None
 TextProperty=None




class ColumnClickEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.ListView.ColumnClick event.

 

 ColumnClickEventArgs(column: int)
 """
 @staticmethod
 def __new__(self,column):
  """ __new__(cls: type,column: int) """
  pass
 Column=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the zero-based index of the column that is clicked.



Get: Column(self: ColumnClickEventArgs) -> int



"""





class AccessibleEvents(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies events that are reported by accessible applications.

 

 enum AccessibleEvents,values: AcceleratorChange (32786),Create (32768),DefaultActionChange (32785),DescriptionChange (32781),Destroy (32769),Focus (32773),HelpChange (32784),Hide (32771),LocationChange (32779),NameChange (32780),ParentChange (32783),Reorder (32772),Selection (32774),SelectionAdd (32775),SelectionRemove (32776),SelectionWithin (32777),Show (32770),StateChange (32778),SystemAlert (2),SystemCaptureEnd (9),SystemCaptureStart (8),SystemContextHelpEnd (13),SystemContextHelpStart (12),SystemDialogEnd (17),SystemDialogStart (16),SystemDragDropEnd (15),SystemDragDropStart (14),SystemForeground (3),SystemMenuEnd (5),SystemMenuPopupEnd (7),SystemMenuPopupStart (6),SystemMenuStart (4),SystemMinimizeEnd (23),SystemMinimizeStart (22),SystemMoveSizeEnd (11),SystemMoveSizeStart (10),SystemScrollingEnd (19),SystemScrollingStart (18),SystemSound (1),SystemSwitchEnd (21),SystemSwitchStart (20),ValueChange (32782)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AcceleratorChange=None
 Create=None
 DefaultActionChange=None
 DescriptionChange=None
 Destroy=None
 Focus=None
 HelpChange=None
 Hide=None
 LocationChange=None
 NameChange=None
 ParentChange=None
 Reorder=None
 Selection=None
 SelectionAdd=None
 SelectionRemove=None
 SelectionWithin=None
 Show=None
 StateChange=None
 SystemAlert=None
 SystemCaptureEnd=None
 SystemCaptureStart=None
 SystemContextHelpEnd=None
 SystemContextHelpStart=None
 SystemDialogEnd=None
 SystemDialogStart=None
 SystemDragDropEnd=None
 SystemDragDropStart=None
 SystemForeground=None
 SystemMenuEnd=None
 SystemMenuPopupEnd=None
 SystemMenuPopupStart=None
 SystemMenuStart=None
 SystemMinimizeEnd=None
 SystemMinimizeStart=None
 SystemMoveSizeEnd=None
 SystemMoveSizeStart=None
 SystemScrollingEnd=None
 SystemScrollingStart=None
 SystemSound=None
 SystemSwitchEnd=None
 SystemSwitchStart=None
 ValueChange=None
 value__=None




"""Wrapper module for pyRevitLabs functionality."""
import logging
import os.path as op
#pylint: disable=W0703,C0302,C0103,W0614,E0401,W0611,C0413
#pylint: disable=superfluous-parens,useless-import-alias
from pyrevit import HOST_APP
from pyrevit.framework import clr
from pyrevit.compat import PY2

# try loading pyrevitlabs
clr.AddReference('Nett')
clr.AddReference('MadMilkman.Ini')
clr.AddReference('OpenMcdf')
clr.AddReference('YamlDotNet')
clr.AddReference('pyRevitLabs.NLog')
clr.AddReference('pyRevitLabs.MahAppsMetro')
# roslyn csharp compiler dependencies are referenced by
# pyRevitLabs.Common thus loading ahead
clr.AddReference('System.Threading.Tasks.Extensions')
clr.AddReference('System.Collections.Immutable')
clr.AddReference('System.Numerics.Vectors')
clr.AddReference('System.Text.Encoding.CodePages')
# Revit, and its builtin addons, ship multiple versions of this assembly
# let's make sure our specific version is loaded
clr.AddReference('System.Runtime.CompilerServices.Unsafe')
clr.AddReference('System.Memory')
# clr.AddReference('System.Memory')
clr.AddReference('System.Reflection.Metadata')
clr.AddReference('Microsoft.CodeAnalysis')
clr.AddReference('Microsoft.CodeAnalysis.CSharp')
# and now
clr.AddReference('pyRevitLabs.Common')
clr.AddReference('pyRevitLabs.CommonCLI')
clr.AddReference('pyRevitLabs.CommonWPF')
clr.AddReference('pyRevitLabs.Emojis')
clr.AddReference('pyRevitLabs.Language')
clr.AddReference('pyRevitLabs.DeffrelDB')
clr.AddReference('pyRevitLabs.TargetApps.Revit')
clr.AddReference('pyRevitLabs.PyRevit')
clr.AddReference('PythonStubsBuilder')
import Nett
import MadMilkman.Ini
import OpenMcdf
import YamlDotNet as libyaml
import pyRevitLabs.MahAppsMetro
from pyRevitLabs import NLog
from pyRevitLabs import Common
from pyRevitLabs import CommonCLI
from pyRevitLabs import CommonWPF
from pyRevitLabs import Emojis
from pyRevitLabs import Language
from pyRevitLabs import DeffrelDB
from pyRevitLabs import TargetApps
from pyRevitLabs import PyRevit
from PythonStubs import PythonStubsBuilder

from pyrevit import coreutils
from pyrevit.coreutils import logger


mlogger = logger.get_logger(__name__)


# setup logger
class PyRevitOutputTarget(NLog.Targets.TargetWithLayout):
    """NLog target to direct log messages to pyRevit output window."""
    def Write(self, asyncLogEvent):
        """Write event handler."""
        try:
            event = asyncLogEvent.LogEvent
            level = self.convert_level(event.Level)
            if mlogger.is_enabled_for(level):
                print(self.Layout.Render(event))    #pylint: disable=E1101
        except Exception as e:
            print(e)

    def convert_level(self, nlog_level):
        """Convert Nlog levels to pything logging levels."""
        if nlog_level == NLog.LogLevel.Fatal:
            return logging.CRITICAL
        elif nlog_level == NLog.LogLevel.Error:
            return logging.ERROR
        elif nlog_level == NLog.LogLevel.Info:
            return logging.INFO
        elif nlog_level == NLog.LogLevel.Debug:
            return logging.DEBUG
        elif nlog_level == NLog.LogLevel.Off:
            return logging.DEBUG
        elif nlog_level == NLog.LogLevel.Trace:
            return logging.DEBUG
        elif nlog_level == NLog.LogLevel.Warn:
            return logging.WARNING


def extract_build_from_exe(proc_path):
    """Extract build number from host .exe file.

    Args:
        proc_path (str): full path of the host .exe file

    Returns:
        (str): build number (e.g. '20170927_1515(x64)')
    """
    # Revit 2021 has a bug on .VersionBuild
    ## it reports identical value as .VersionNumber
    pinfo = TargetApps.Revit.RevitProductData.GetBinaryProductInfo(proc_path)
    return "{}({})".format(pinfo.build, pinfo.target) \
        if pinfo.build else "20000101_0000(x64)"


# activate binding resolver
if HOST_APP.is_older_than(2019):
    PyRevit.PyRevitBindings.ActivateResolver()

# configure NLog
#pylint: disable=W0201
if PY2:
    config = NLog.Config.LoggingConfiguration()
    target = PyRevitOutputTarget()
    target.Name = __name__
    target.Layout = "${level:uppercase=true} [${logger}] ${message}"
    config.AddTarget(__name__, target)
    config.AddRuleForAllLevels(target)
    NLog.LogManager.Configuration = config

    for rule in NLog.LogManager.Configuration.LoggingRules:
        rule.EnableLoggingForLevel(NLog.LogLevel.Info)
        rule.EnableLoggingForLevel(NLog.LogLevel.Debug)

    nlog_mlogger = NLog.LogManager.GetLogger(__name__)



class AttributeCollection(object,ICollection,IEnumerable):
 """
 Represents a collection of attributes.

 

 AttributeCollection(*attributes: Array[Attribute])
 """
 def Contains(self,*__args):
  """
  Contains(self: AttributeCollection,attributes: Array[Attribute]) -> bool

  

   Determines whether this attribute collection contains all the specified attributes in the 

    attribute array.

  

  

   attributes: An array of type System.Attribute to find in the collection.

   Returns: true if the collection contains all the attributes; otherwise,false.

  Contains(self: AttributeCollection,attribute: Attribute) -> bool

  

   Determines whether this collection of attributes has the specified attribute.

  

   attribute: An System.Attribute to find in the collection.

   Returns: true if the collection contains the attribute or is the default attribute for the type of 

    attribute; otherwise,false.
  """
  pass
 def CopyTo(self,array,index):
  """
  CopyTo(self: AttributeCollection,array: Array,index: int)

   Copies the collection to an array,starting at the specified index.

  

   array: The System.Array to copy the collection to.

   index: The index to start from.
  """
  pass
 @staticmethod
 def FromExisting(existing,newAttributes):
  """
  FromExisting(existing: AttributeCollection,*newAttributes: Array[Attribute]) -> AttributeCollection

  

   Creates a new System.ComponentModel.AttributeCollection from an existing 

    System.ComponentModel.AttributeCollection.

  

  

   existing: An System.ComponentModel.AttributeCollection from which to create the copy.

   newAttributes: An array of type System.Attribute that provides the attributes for this collection. Can be null.

   Returns: A new System.ComponentModel.AttributeCollection that is a copy of existing.
  """
  pass
 def GetDefaultAttribute(self,*args):
  """
  GetDefaultAttribute(self: AttributeCollection,attributeType: Type) -> Attribute

  

   Returns the default System.Attribute of a given System.Type.

  

   attributeType: The System.Type of the attribute to retrieve.

   Returns: An System.Attribute.
  """
  pass
 def GetEnumerator(self):
  """
  GetEnumerator(self: AttributeCollection) -> IEnumerator

  

   Gets an enumerator for this collection.

   Returns: An enumerator of type System.Collections.IEnumerator.
  """
  pass
 def Matches(self,*__args):
  """
  Matches(self: AttributeCollection,attributes: Array[Attribute]) -> bool

  

   Determines whether the attributes in the specified array are the same as the attributes in the 

    collection.

  

  

   attributes: An array of System.CodeDom.MemberAttributes to compare with the attributes in this collection.

   Returns: true if all the attributes in the array are contained in the collection and have the same values 

    as the attributes in the collection; otherwise,false.

  

  Matches(self: AttributeCollection,attribute: Attribute) -> bool

  

   Determines whether a specified attribute is the same as an attribute in the collection.

  

   attribute: An instance of System.Attribute to compare with the attributes in this collection.

   Returns: true if the attribute is contained within the collection and has the same value as the attribute 

    in the collection; otherwise,false.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y]x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 @staticmethod
 def __new__(self,attributes):
  """
  __new__(cls: type,*attributes: Array[Attribute])

  __new__(cls: type)
  """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 Attributes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the attribute collection.



"""

 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of attributes.



Get: Count(self: AttributeCollection) -> int



"""


 Empty=None

